@@ -47,8 +47,10 @@ assistant_finished_cb (GnomeDbDsnAssistant *assistant, gboolean error, gpointer
 
 	if (!error) {
 		dsn_info = gnome_db_dsn_assistant_get_dsn (assistant);
-		if (dsn_info)
-			gda_config_save_data_source_info (dsn_info);
+		if (dsn_info) {
+			if (! gda_config_define_dsn (dsn_info, NULL)) 
+				gnome_db_show_error (NULL, _("Could not declare new data source"));
+		}
 		else
 			gnome_db_show_error (NULL, _("No valid data source info was created"));
 	}
@@ -115,7 +117,7 @@ file_delete_cb (GtkAction *action, gpointer user_data)
 static void
 window_closed_cb (GtkAction *action, gpointer user_data)
 {
-	gnome_db_main_quit ();
+	gtk_main_quit ();
 }
 
 static void
@@ -233,7 +235,7 @@ prepare_menu (GtkBox *vbox, GtkWidget *nb)
 
 
 static void
-create_main_window (gpointer user_data)
+create_main_window (void)
 {
 	GtkWidget *window, *vbox;
 	GtkWidget *nb;
@@ -285,10 +287,13 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
-	gnome_db_init ("gnome-database-properties", PACKAGE_VERSION, argc, argv);
+	gnome_db_init ();
+	gtk_init (&argc, &argv);
+
+	create_main_window ();
 
 	/* application loop */
-	gnome_db_main_run ((GdaInitFunc) create_main_window, NULL);
+	gtk_main ();
 
 	return 0;
 }
@@ -297,10 +302,15 @@ static void
 dsn_selection_changed_cb (GnomeDbRawGrid *dbrawgrid, gboolean row_selected, gpointer data)
 {
 	GtkAction *action;
+	GList *sel;
+
 	action = gtk_action_group_get_action (actions, "DatabaseProperties");
 	g_object_set (G_OBJECT (action), "sensitive", row_selected, NULL);
+
+	sel = gnome_db_raw_grid_get_selection (dbrawgrid);
 	action = gtk_action_group_get_action (actions, "DatabaseDelete");
-	g_object_set (G_OBJECT (action), "sensitive", row_selected, NULL);
+	g_object_set (G_OBJECT (action), "sensitive", sel ? TRUE : FALSE, NULL);
+	g_list_free (sel);
 }
 
 static void
