@@ -6,20 +6,21 @@
 
 #include <libgnomedb/libgnomedb.h>
 
-extern GdaDict *demo_dict;
+extern GdaConnection *demo_cnc;
+extern GdaSqlParser *demo_parser;
 static GtkWidget *window = NULL;
 
 GtkWidget *
 do_linked_model_param (GtkWidget *do_widget)
 {  
 	if (!window) {
-                GdaQuery *query;
+                GdaStatement *stmt;
 		GtkWidget *vbox;
 		GtkWidget *label;
 		GdaDataModel *cust_model, *sr_model;
 		GtkWidget *form, *grid;
-		GdaParameterList *params;
-		GdaParameter *param;
+		GdaSet *params;
+		GdaHolder *param;
 
 		window = gtk_dialog_new_with_buttons ("GdaDataModel depending on a parameter",
 						      GTK_WINDOW (do_widget),
@@ -47,24 +48,24 @@ do_linked_model_param (GtkWidget *do_widget)
 		gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
 
 		/* create a data model for the salesrep */
-		query = gda_query_new_from_sql (demo_dict, "SELECT id, name FROM salesrep", NULL);
-		sr_model = gda_data_model_query_new (query);
-		g_object_unref (query);
+		stmt = gda_sql_parser_parse_string (demo_parser, "SELECT id, name FROM salesrep", NULL, NULL);
+		sr_model = gda_data_model_query_new (demo_cnc, stmt);
+		g_object_unref (stmt);
 		
 		/* create a data model for the customers, and get a list of parameters */
-                query = gda_query_new_from_sql (demo_dict, 
+		stmt = gda_sql_parser_parse_string (demo_parser, 
 						"SELECT c.id, c.name, s.name AS \"SalesRep\""
 						"FROM customers c "
 						"LEFT JOIN salesrep s ON (s.id=c.default_served_by) "
-						"WHERE s.id=##/*name:'SalesRep' type:gint descr:'Sales person'*/ "
-						"OR s.id IS NULL", NULL);
-		cust_model = gda_data_model_query_new (query);
-		g_object_unref (query);
+						"WHERE s.id = ##/*name:'SalesRep' type:gint descr:'Sales person'*/ "
+						    "OR s.id IS NULL", NULL, NULL);
+		cust_model = gda_data_model_query_new (demo_cnc, stmt);
+		g_object_unref (stmt);
 		params = gda_data_model_query_get_parameter_list (GDA_DATA_MODEL_QUERY (cust_model));
 
 		/* restrict the c.default_served_by field in the grid to be within the sr_model */
-		param = gda_parameter_list_find_param (params, "SalesRep");
-		g_assert (gda_parameter_restrict_values (param, sr_model, 0, NULL));
+		param = gda_set_get_holder (params, "SalesRep");
+		g_assert (gda_holder_set_source_model (param, sr_model, 0, NULL));
 
 
 		/* create a basic form to set the values in params */
