@@ -1,6 +1,6 @@
 /* gnome-db-sql-console.c
  *
- * Copyright (C) 2005 - 2007 Vivien Malerba
+ * Copyright (C) 2005 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -22,6 +22,7 @@
 #include "gnome-db-sql-console.h"
 #include <gdk/gdkkeysyms.h>
 #include <libgda/libgda.h>
+#include <sql-parser/gda-sql-parser.h>
 #include <glib/gi18n-lib.h>
 #include "gnome-db-basic-form.h"
 #include "extra_marshal.h"
@@ -41,8 +42,7 @@ static void gnome_db_sql_console_get_property (GObject *object,
 
 enum {
 	PROP_0,
-
-	PROP_DICT,
+	PROP_CNC,
 	PROP_MESSAGE
 };
 
@@ -68,7 +68,8 @@ typedef struct {
 
 struct _GnomeDbSqlConsolePriv
 {
-	GdaDict       *dict;
+	GdaConnection *cnc;
+	GdaSqlParser  *parser;
 	GtkTextBuffer *buffer;
 
 	GList         *hist_items; /* list of HistItem elements */
@@ -148,10 +149,9 @@ gnome_db_sql_console_class_init (GnomeDbSqlConsoleClass * class)
 	object_class->set_property = gnome_db_sql_console_set_property;
 	object_class->get_property = gnome_db_sql_console_get_property;
 
-	g_object_class_install_property (object_class, PROP_DICT,
-	                                 g_param_spec_object("dict", NULL, NULL,
-	                                                     GDA_TYPE_DICT,
-	                                                     G_PARAM_READWRITE | G_PARAM_CONSTRUCT_ONLY));
+	g_object_class_install_property (object_class, PROP_CNC,
+	                                 g_param_spec_object ("cnc", NULL, NULL,
+							      GDA_TYPE_CONNECTION, G_PARAM_READWRITE));
 
 	g_object_class_install_property (object_class, PROP_MESSAGE,
 	                                 g_param_spec_string("message", NULL, NULL, NULL,
@@ -172,45 +172,34 @@ gnome_db_sql_console_class_init (GnomeDbSqlConsoleClass * class)
 
 static void gnome_db_sql_console_initialize (GnomeDbSqlConsole *console);
 static void add_prompt (GnomeDbSqlConsole *console);
-static void gda_dict_weak_notify (GnomeDbSqlConsole *console, GdaDict *dict);
 
 static void
 gnome_db_sql_console_init (GnomeDbSqlConsole * wid)
 {
 	wid->priv = g_new0 (GnomeDbSqlConsolePriv, 1);
-	wid->priv->dict = NULL;
+	wid->priv->cnc = NULL;
+	wid->priv->parser = NULL;
 
-	gnome_db_sql_console_initialize(wid);
+	gnome_db_sql_console_initialize (wid);
 }
 
 /**
  * gnome_db_sql_console_new
- * @dict: a #GdaDict object
+ * @cnc: a #GdaConnection connection, or %NULL
  *
  * Creates a new #GnomeDbSqlConsole widget 
  *
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_sql_console_new (GdaDict *dict, const gchar *message)
+gnome_db_sql_console_new (GdaConnection *cnc, const gchar *message)
 {
-	GObject *obj;
+	g_return_val_if_fail (!cnc || GDA_IS_CONNECTION (cnc), NULL);
 
-	g_return_val_if_fail (!dict || GDA_IS_DICT (dict), NULL);
-
-	obj = g_object_new (GNOME_DB_TYPE_SQL_CONSOLE, "dict", dict, "message", message, NULL);
-
-	return GTK_WIDGET (obj);
+	return (GtkWidget*) g_object_new (GNOME_DB_TYPE_SQL_CONSOLE, "cnc", cnc, "message", message, NULL);
 }
 
 
-static void
-gda_dict_weak_notify (GnomeDbSqlConsole *console, GdaDict *dict)
-{
-	/* Tell that we don't need to weak unref the GdaDict */
-	console->priv->dict = NULL;
-}
-
 static void
 gnome_db_sql_console_dispose (GObject *object)
 {
@@ -224,10 +213,14 @@ gnome_db_sql_console_dispose (GObject *object)
 		/* history */
 		gnome_db_sql_console_clear_history (console);
 		
-		/* Weak unref the GdaDict is necessary */
-		if (console->priv->dict)
-			g_object_weak_unref (G_OBJECT (console->priv->dict),
-					     (GWeakNotify) gda_dict_weak_notify, console);
+		if (console->priv->cnc) {
+			g_object_unref (console->priv->cnc);
+			console->priv->cnc = NULL;
+		}
+		if (console->priv->parser) {
+			g_object_unref (console->priv->parser);
+			console->priv->parser = NULL;
+		}
 		
 		/* the private area itself */
 		g_free (console->priv);
@@ -238,28 +231,28 @@ gnome_db_sql_console_dispose (GObject *object)
 	parent_class->dispose (object);
 }
 
-static void gnome_db_sql_console_set_property (GObject *object,
-                                               guint param_id,
-                                               const GValue *value,
-                                               GParamSpec *pspec)
+static void
+gnome_db_sql_console_set_property (GObject *object,
+				   guint param_id,
+				   const GValue *value,
+				   GParamSpec *pspec)
 {
 	GnomeDbSqlConsole *console;
 	GtkTextIter iter;
-	GdaDict *dict;
 
 	console = GNOME_DB_SQL_CONSOLE(object);
 
 	switch(param_id) {
-	case PROP_DICT:
-		dict = GDA_DICT (g_value_get_object (value));
-		console->priv->dict = dict;
-		if (dict)
-		{
-			g_object_weak_ref (G_OBJECT (dict),
-					   (GWeakNotify) gda_dict_weak_notify,
-			                   console);
+	case PROP_CNC:
+		if (console->priv->cnc) 
+			g_object_unref (console->priv->cnc);
+		if (console->priv->parser) {
+			g_object_unref (console->priv->parser);
+			console->priv->parser = NULL;
 		}
-
+		console->priv->cnc = GDA_CONNECTION (g_value_get_object (value));
+		if (console->priv->cnc) 
+			g_object_ref (console->priv->cnc);
 		break;
 	case PROP_MESSAGE:
 		/* initial text */
@@ -284,8 +277,8 @@ static void gnome_db_sql_console_get_property (GObject *object,
 	console = GNOME_DB_SQL_CONSOLE(object);
 
 	switch(param_id) {
-	case PROP_DICT:
-		g_value_set_object (value, G_OBJECT (console->priv->dict));
+	case PROP_CNC:
+		g_value_set_object (value, G_OBJECT (console->priv->cnc));
 		break;
 	}
 }
@@ -306,7 +299,6 @@ gnome_db_sql_console_initialize (GnomeDbSqlConsole *console)
 {
 	GtkTextBuffer *buffer;
 	PangoFontDescription *font_desc;
-	GtkTextIter iter;
 
 	/* widget init */
 	gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW (console), GTK_WRAP_NONE);
@@ -520,14 +512,14 @@ view_event_cb (GtkWidget *widget, GdkEvent *event, GnomeDbSqlConsole *console)
 }
 
 static gint
-fill_context_in_dialog (GnomeDbSqlConsole *console, GdaParameterList *context)
+fill_context_in_dialog (GnomeDbSqlConsole *console, GdaSet *context)
 {
 	GtkWidget *dlg;
 	GnomeDbBasicForm *sform;
 	GtkWidget *parent_window;
 	gint result;
 
-	if (!context || gda_parameter_list_is_valid (context)) 
+	if (!context || gda_set_is_valid (context)) 
 		return GTK_RESPONSE_ACCEPT;
 	
 	parent_window = gtk_widget_get_toplevel (GTK_WIDGET (console));
@@ -556,11 +548,11 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 {
 	GdaConnection *cnc;
 	GtkTextIter iter;
-	GdaQuery *query;
-	GdaParameterList *dset;
+	GdaBatch *batch;
 	GError *error = NULL;
-	GdaDataModel *model;
 	gchar *sigret = NULL;
+	const gchar *remain;
+	const GSList *stmt_list;
 
 #ifdef debug_signal
 	g_print (">> 'SQL_ENTERED' from %s\n", __FUNCTION__);
@@ -578,114 +570,119 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 		g_free (sigret);
 	}
 
-	if (!console->priv->dict) {
+	if (!console->priv->cnc) {
 		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, "result-ok", NULL);
 		return;
 	}
 
-	cnc = gda_dict_get_connection (console->priv->dict);
-	if (!cnc) {
-		g_warning ("No connection associated to dictionary, can't run query");
-		return;
-	}
-	if (! gda_connection_is_opened (cnc)) {
-		g_warning ("Connection closed, can't run query");
+	if (!gda_connection_is_opened (console->priv->cnc)) {
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "Connection is closed!\n", 
+							  1, "result-error", NULL);
 		return;
 	}
 	
 	/* run query */
-	query = gda_query_new_from_sql (console->priv->dict, sql, NULL);
-	dset = gda_query_get_parameter_list (query);
-	if (fill_context_in_dialog (console, dset) != GTK_RESPONSE_ACCEPT) {
-		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, _("Cancelled"), -1, 
-							  "result-error", NULL);
-		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, 
-							  "result-error", NULL);
+	if (!console->priv->parser) {
+		console->priv->parser = gda_connection_create_parser (console->priv->cnc);
+		if (!console->priv->parser)
+			console->priv->parser = gda_sql_parser_new ();
+	}
+	
+	batch = gda_sql_parser_parse_string_as_batch (console->priv->parser, sql, &remain, NULL);
+	if (!batch) {
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "Syntax error in SQL!\n", 
+							  1, "result-error", NULL);
+		return;
+	}
+	else if (remain) {
+		gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "Syntax error in SQL!\n", 
+							  1, "result-error", NULL);
+		g_object_unref (batch);
+		return;
 	}
-	else {
-		gchar *rsql;
 
-		if (dset && dset->parameters) {
-			/* dump param's values */
-			GString *string;
-			GSList *list;
-
-			string = g_string_new ("");
-			list = dset->parameters;
-			while (list) {
-				gchar *str;
-				const GValue *value;
-				value = gda_parameter_get_value (GDA_PARAMETER (list->data));
-				str = gda_value_stringify ((GValue *) value);
-				g_string_append_printf (string, "%s => %s\n", 
-							gda_object_get_name (GDA_OBJECT (list->data)), str);
-				g_free (str);
-				list = g_slist_next (list);
-			}
-			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, string->str, -1, 
-								  "params_dump", NULL);
-			g_string_free (string, TRUE);
+	/* FIXME: compute a unique GdaSet for all the statements in @batch */
+	hist_item->sql = g_strdup (sql);
+	for (stmt_list = gda_batch_get_statements (batch); stmt_list; stmt_list = stmt_list->next) {
+		GdaStatement *stmt = GDA_STATEMENT (stmt_list->data);
+		GdaSet *params;
+		if (gda_statement_is_useless (stmt))
+			continue;
+		if (!gda_statement_get_parameters (stmt, &params, NULL)) {
+			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "Syntax error in SQL!\n", 
+								  1, "result-error", NULL);
+			g_object_unref (batch);
+			return;
 		}
 
-		/* run query */
-		rsql = gda_renderer_render_as_sql (GDA_RENDERER (query), dset, NULL, 0, &error);
-		if (rsql) {
-			GdaCommand *cmde;
-			GList *reslist;
-
-			cmde = gda_command_new (rsql, GDA_COMMAND_TYPE_SQL, GDA_COMMAND_OPTION_STOP_ON_ERRORS);
-			reslist = gda_connection_execute_command (cnc, cmde, NULL, &error);
-			gda_command_free (cmde);
-			if (reslist) {
-				GList *list;
-				for (list = reslist; list; list = list->next) {
-					GString *string;
+		if (fill_context_in_dialog (console, params) != GTK_RESPONSE_ACCEPT) {
+			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, _("Cancelled"), -1, 
+								  "result-error", NULL);
+			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, 
+								  "result-error", NULL);
+		}
+		else {
+			if (params && params->holders) {
+				/* dump param's values */
+				GString *string;
+				GSList *list;
 				
-					string = g_string_new ("");
-
-					model = (GdaDataModel *) (list->data);
-					if (GDA_IS_DATA_MODEL (model)) {
-						gchar *str1;
-						
-						str1 = gda_data_model_dump_as_string (model);
-						g_string_append (string, str1);
-						g_free (str1);
-					}
-					else if (GDA_IS_PARAMETER_LIST (model)) {
-						GdaParameter *param;
-
-						param = gda_parameter_list_find_param (GDA_PARAMETER_LIST (model), 
-										       "IMPACTED_ROWS");
-						if (param) {
-							const GValue *value;
-							gint rows;
-
-							value = gda_parameter_get_value (param);
-							rows = g_value_get_int (value);
-							g_string_append_printf (string, 
-										ngettext("%d row impacted\n",
-										"%d rows impacted\n", rows), rows);
-						} 
-					}
-					
-					if (list->data)
-						g_object_unref (list->data);
-
-					if (list == reslist)
-						gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, 
-											  &iter, 
-											  "\n", 1, "result-ok", NULL);
-					gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter,
-										  string->str, -1,"result-ok", NULL);
-					gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, 
-										  "\n\n", 2, "result-ok", NULL);
-					g_string_free (string, TRUE);
+				string = g_string_new ("");
+				for (list = params->holders; list; list = list->next) {
+					gchar *str;
+					const GValue *value;
+					value = gda_holder_get_value (GDA_HOLDER (list->data));
+					str = gda_value_stringify ((GValue *) value);
+					g_string_append_printf (string, "%s => %s\n", 
+								(gchar *) g_object_get_data (G_OBJECT (list->data), "name"),
+								str);
+					g_free (str);
 				}
-				g_list_free (reslist);
+				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, string->str, -1, 
+									  "params_dump", NULL);
+				g_string_free (string, TRUE);
+			}
+
+			/* run query */
+			GObject *res;
+			res = gda_connection_statement_execute (console->priv->cnc, stmt, params, 
+								GDA_STATEMENT_MODEL_RANDOM_ACCESS, NULL, &error);
+			if (res) {
+				GString *string;
+				string = g_string_new ("");
+
+				if (GDA_IS_DATA_MODEL (res)) {
+					gchar *str1;
+					
+					str1 = gda_data_model_dump_as_string ((GdaDataModel*) res);
+					g_string_append (string, str1);
+					g_free (str1);
+				}
+				else if (GDA_IS_SET (res)) {
+					GdaHolder *ir;
+					ir = gda_set_get_holder (GDA_SET (res), "IMPACTED_ROWS");
+					if (ir) {
+						const GValue *value;
+						gint rows;
+						value = gda_holder_get_value (ir);
+						rows = g_value_get_int (value);
+						g_string_append_printf (string, 
+									ngettext ("%d row impacted\n",
+										  "%d rows impacted\n", rows), rows);
+					} 
+				}
+				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, 
+									  &iter, 
+									  "\n", 1, "result-ok", NULL);
+				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter,
+									  string->str, -1,"result-ok", NULL);
+				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, 
+									  "\n\n", 2, "result-ok", NULL);
+				g_string_free (string, TRUE);
+				g_object_unref (res);
 			}
 			else {
 				gchar *str = NULL;
-				gboolean cmde_ok = FALSE;
 				if (error) {
 					if (error->message)
 						str = g_strdup (error->message);
@@ -697,35 +694,19 @@ run_sql_cmde (GnomeDbSqlConsole *console, HistItem *hist_item, const gchar *sql)
 					str = compute_events_as_string (cnc);
 					if (!str)
 						str = g_strdup (_("Ok."));
-					
-					cmde_ok = TRUE;
 				}
 				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, 
-									  cmde_ok ? "result-ok" : "result-error", NULL);
+									  "result-error", NULL);
 				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, str, -1, 
-									  cmde_ok ? "result-ok" : "result-error", NULL);
+									  "result-error", NULL);
 				gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, 
-									  cmde_ok ? "result-ok" : "result-error", NULL);
+									  "result-error", NULL);
 				g_free (str);
+				break; /* stop statements execution */
 			}
-			hist_item->sql = rsql;
-		}
-		else {
-			gchar *str = NULL;
-			if (error && error->message)
-				str = g_strdup (error->message);
-			else
-				str = g_strdup (_("Error while computing SQL statement"));
-			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n", 1, "result-error", NULL);
-			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, _("Cancelled"), -1, "result-error", NULL);
-			gtk_text_buffer_insert_with_tags_by_name (console->priv->buffer, &iter, "\n\n", 2, "result-error", NULL);
-			g_free (str);
 		}
 	}
-
-	if (dset)
-		g_object_unref (dset);
-	g_object_unref (query);
+	g_object_unref (batch);
 }
 
 static gchar *
