@@ -1,6 +1,6 @@
 /* gnome-db-canvas-db-relations.h
  *
- * Copyright (C) 2004 Vivien Malerba
+ * Copyright (C) 2004 - 2007 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -53,7 +53,12 @@ struct _GnomeDbCanvasDbRelationsClass
 /* generic widget's functions */
 GType            gnome_db_canvas_db_relations_get_type        (void) G_GNUC_CONST;
 
-GtkWidget       *gnome_db_canvas_db_relations_new             (GdaDict *dict, GdaGraph *graph);
+GtkWidget       *gnome_db_canvas_db_relations_new             (GdaMetaStore *store);
+
+GnomeDbCanvasTable *gnome_db_canvas_db_relations_get_table_item  (GnomeDbCanvasDbRelations *canvas, GdaMetaTable *table);
+GnomeDbCanvasTable *gnome_db_canvas_db_relations_add_table_item  (GnomeDbCanvasDbRelations *canvas, 
+							    const GValue *table_catalog, const GValue *table_schema,
+							    const GValue *table_name);
 
 G_END_DECLS
 
