@@ -1,7 +1,7 @@
 /* gnome-db-data-cell-renderer-combo.c
  *
  * Copyright (C) 2000  Red Hat, Inc.,  Jonathan Blandford <jrb@redhat.com>
- * Copyright (C) 2003 - 2006 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2003 - 2008 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -87,14 +87,13 @@ enum {
 
 struct _GnomeDbDataCellRendererComboPrivate
 {
-	GdaParameterList       *paramlist;
-	GdaParameterListSource *source;
-	guint                   focus_out_id;
-
-	guint                   attributes;
-	gboolean                to_be_deleted;
-	gboolean                set_default_if_invalid;
-	gboolean                show_expander;
+	GdaSet       *paramlist;
+	GdaSetSource *source;
+	guint         focus_out_id;
+	guint         attributes;
+	gboolean      to_be_deleted;
+	gboolean      set_default_if_invalid;
+	gboolean      show_expander;
 };
 
 
@@ -188,10 +187,10 @@ gnome_db_data_cell_renderer_combo_class_init (GnomeDbDataCellRendererComboClass
                                                                (G_PARAM_READABLE | G_PARAM_WRITABLE)));
   
 	g_object_class_install_property (object_class, PROP_PARAMLIST,
-					 g_param_spec_object ("param_list", NULL, NULL, GDA_TYPE_PARAMETER_LIST,
+					 g_param_spec_object ("param_list", NULL, NULL, GDA_TYPE_SET,
                                                                (G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE)));
 
-	/* Ideally, GdaParameterListSource would be a boxed type, but it is not yet, so we use g_param_spec_pointer(). */
+	/* Ideally, GdaSetSource would be a boxed type, but it is not yet, so we use g_param_spec_pointer(). */
 	g_object_class_install_property (object_class, PROP_PARAMLISTSOURCE,
 					 g_param_spec_pointer ("param_list_source", NULL, NULL,
                                                                (G_PARAM_CONSTRUCT_ONLY | G_PARAM_WRITABLE)));
@@ -210,21 +209,13 @@ gnome_db_data_cell_renderer_combo_class_init (GnomeDbDataCellRendererComboClass
 
 }
 
-static void
-paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbDataCellRendererCombo *datacell)
-{
-	g_signal_handlers_disconnect_by_func (paramlist, G_CALLBACK (paramlist_destroyed_cb), datacell);
-	datacell->priv->paramlist = NULL;
-	datacell->priv->source = NULL;
-}
-
 static void
 gnome_db_data_cell_renderer_combo_dispose (GObject *object)
 {
 	GnomeDbDataCellRendererCombo *datacell = GNOME_DB_DATA_CELL_RENDERER_COMBO (object);
 
 	if (datacell->priv->paramlist) {
-		paramlist_destroyed_cb (datacell->priv->paramlist, datacell);
+		g_object_unref (datacell->priv->paramlist);
 		datacell->priv->paramlist = NULL;
 	}
 
@@ -350,8 +341,6 @@ gnome_db_data_cell_renderer_combo_set_property (GObject *object,
 	case PROP_VALUES_DISPLAY:
 		if (value) {
 			GList *gvalues = g_value_get_pointer (value);
-			GValue *value;
-			gint i;
 			gchar *str;
 
 			g_assert (g_list_length (gvalues) == datacell->priv->source->shown_n_cols);
@@ -380,15 +369,14 @@ gnome_db_data_cell_renderer_combo_set_property (GObject *object,
 		if (datacell->priv->paramlist)
 			g_object_unref (datacell->priv->paramlist);
 
-		datacell->priv->paramlist = GDA_PARAMETER_LIST(g_value_get_object(value));
+		datacell->priv->paramlist = GDA_SET(g_value_get_object(value));
 		if(datacell->priv->paramlist)
 			g_object_ref(datacell->priv->paramlist);
 
-		gda_object_connect_destroy (GDA_OBJECT (datacell->priv->paramlist), 
-				    G_CALLBACK (paramlist_destroyed_cb), datacell);
+		g_object_ref (G_OBJECT (datacell->priv->paramlist)); 
 		break;
 	case PROP_PARAMLISTSOURCE:
-		datacell->priv->source = GDA_PARAMETER_LIST_SOURCE(g_value_get_pointer(value));
+		datacell->priv->source = GDA_SET_SOURCE(g_value_get_pointer(value));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -437,8 +425,8 @@ render_text_to_display_from_values (GList *values)
 
 /**
  * gnome_db_data_cell_renderer_combo_new
- * @paramlist: a #GdaParameterList object
- * @source: a #GdaParameterListSource structure listed in @paramlist->sources_list
+ * @paramlist: a #GdaSet object
+ * @source: a #GdaSetSource structure listed in @paramlist->sources_list
  * 
  * Creates a new #GnomeDbDataCellRendererCombo which will fill the parameters listed in
  * @source->nodes with values available from @source->data_model.
@@ -446,11 +434,11 @@ render_text_to_display_from_values (GList *values)
  * Return value: the new cell renderer
  **/
 GtkCellRenderer *
-gnome_db_data_cell_renderer_combo_new (GdaParameterList *paramlist, GdaParameterListSource *source)
+gnome_db_data_cell_renderer_combo_new (GdaSet *paramlist, GdaSetSource *source)
 {
 	GObject *obj;
 
-	g_return_val_if_fail (GDA_IS_PARAMETER_LIST (paramlist), NULL);
+	g_return_val_if_fail (GDA_IS_SET (paramlist), NULL);
 	g_return_val_if_fail (source, NULL);
 	g_return_val_if_fail (g_slist_find (paramlist->sources_list, source), NULL);
 
