@@ -46,14 +46,14 @@ static void layout_spec_free (GnomeDbFormLayoutSpec *spec);
 static void gnome_db_basic_form_fill (GnomeDbBasicForm *form);
 static void gnome_db_basic_form_clean (GnomeDbBasicForm *form);
 
-static void paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
-static void paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form);
-static void paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
-static void paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form);
+static void get_rid_of_set (GdaSet *paramlist, GnomeDbBasicForm *form);
+static void paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbBasicForm *form);
+static void paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbBasicForm *form);
+static void paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbBasicForm *form);
 
 static void entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
 static void entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form);
-static void parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry);
+static void parameter_changed_cb (GdaHolder *param, GnomeDbDataEntry *entry);
 
 static void mark_not_null_entry_labels (GnomeDbBasicForm *form, gboolean show_mark);
 enum
@@ -76,7 +76,7 @@ enum
 
 struct _GnomeDbBasicFormPriv
 {
-	GdaParameterList       *paramlist;/* parameters */
+	GdaSet                 *set;
 	gulong                 *signal_ids; /* array of signal ids */
 
 	GSList                 *entries;/* list of GnomeDbDataEntry widgets */
@@ -139,7 +139,7 @@ gnome_db_basic_form_class_init (GnomeDbBasicFormClass * class)
 			      G_STRUCT_OFFSET (GnomeDbBasicFormClass, param_changed),
 			      NULL, NULL,
 			      gnome_db_marshal_VOID__OBJECT_BOOLEAN, G_TYPE_NONE, 2,
-			      GDA_TYPE_PARAMETER, G_TYPE_BOOLEAN);
+			      GDA_TYPE_HOLDER, G_TYPE_BOOLEAN);
 	gnome_db_basic_form_signals[ACTIVATED] =
 		g_signal_new ("activated",
 			      G_TYPE_FROM_CLASS (object_class),
@@ -185,7 +185,7 @@ static void
 gnome_db_basic_form_init (GnomeDbBasicForm * wid)
 {
 	wid->priv = g_new0 (GnomeDbBasicFormPriv, 1);
-	wid->priv->paramlist = NULL;
+	wid->priv->set = NULL;
 	wid->priv->entries = NULL;
 	wid->priv->not_null_labels = NULL;
 	wid->priv->layout_spec = NULL;
@@ -208,7 +208,7 @@ static void widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form);
 
 /**
  * gnome_db_basic_form_new
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  *
  * Creates a new #GnomeDbBasicForm widget using all the parameters provided in @paramlist.
  *
@@ -218,7 +218,7 @@ static void widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form);
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_basic_form_new (GdaParameterList *paramlist)
+gnome_db_basic_form_new (GdaSet *paramlist)
 {
 	GObject *obj;
 
@@ -229,7 +229,7 @@ gnome_db_basic_form_new (GdaParameterList *paramlist)
 
 /**
  * gnome_db_basic_form_new_custom
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  * @glade_file: a Glade XML file name
  * @root_element: the name of the top-most widget in @glade_file to use in the new form
  * @form_prefix: the prefix used to look for widgets to add entries in
@@ -242,7 +242,7 @@ gnome_db_basic_form_new (GdaParameterList *paramlist)
  * Returns: the new widget
  */
 GtkWidget *
-gnome_db_basic_form_new_custom (GdaParameterList *paramlist, const gchar *glade_file, 
+gnome_db_basic_form_new_custom (GdaSet *paramlist, const gchar *glade_file, 
 				const gchar *root_element, const gchar *form_prefix)
 {
 	GnomeDbFormLayoutSpec spec;
@@ -273,15 +273,15 @@ widget_shown_cb (GtkWidget *wid, GnomeDbBasicForm *form)
 }
 
 static void
-paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
+get_rid_of_set (GdaSet *paramlist, GnomeDbBasicForm *form)
 {
 	GSList *list;
 	gint i = 0;
 
-	g_assert (paramlist == form->priv->paramlist);
+	g_assert (paramlist == form->priv->set);
 
 	/* disconnect from parameters */
-	list = form->priv->paramlist->parameters;;
+	list = form->priv->set->holders;;
 	while (list) {
 		g_signal_handler_disconnect (G_OBJECT (list->data), form->priv->signal_ids[i]);
 		
@@ -292,8 +292,6 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 	form->priv->signal_ids = NULL;
 
 	/* unref the paramlist */
-	g_signal_handlers_disconnect_by_func (paramlist,
-					      G_CALLBACK (paramlist_destroyed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_public_data_changed_cb), form);
 	g_signal_handlers_disconnect_by_func (paramlist,
@@ -301,8 +299,8 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 	g_signal_handlers_disconnect_by_func (paramlist,
 					      G_CALLBACK (paramlist_param_attr_changed_cb), form);
 
-	g_object_unref (form->priv->paramlist);
-	form->priv->paramlist = NULL;
+	g_object_unref (form->priv->set);
+	form->priv->set = NULL;
 
 	/* render all the entries non sensitive */
 	list = form->priv->entries;
@@ -313,7 +311,7 @@ paramlist_destroyed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
 }
 
 static void
-paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm *form)
+paramlist_public_data_changed_cb (GdaSet *paramlist, GnomeDbBasicForm *form)
 {
 	/* here we want to re-define all the data entry widgets */
 	gnome_db_basic_form_clean (form);
@@ -321,14 +319,14 @@ paramlist_public_data_changed_cb (GdaParameterList *paramlist, GnomeDbBasicForm
 }
 
 static void
-paramlist_param_plugin_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
+paramlist_param_plugin_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbBasicForm *form)
 {
 	/* TODO: be more specific and change only the entry corresponding to @param */
 	paramlist_public_data_changed_cb (paramlist, form);
 }
 
 static void
-paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *param, GnomeDbBasicForm *form)
+paramlist_param_attr_changed_cb (GdaSet *paramlist, GdaHolder *param, GnomeDbBasicForm *form)
 {
 	GtkWidget *entry;
 
@@ -337,12 +335,13 @@ paramlist_param_attr_changed_cb (GdaParameterList *paramlist, GdaParameter *para
 		gboolean toset;
 		guint attrs = 0;
 		guint mask = 0;
+		const GValue *defv;
 
-		toset = gda_parameter_get_exists_default_value (param);
-		attrs |= toset ? GDA_VALUE_ATTR_CAN_BE_DEFAULT : 0;
+		defv = gda_holder_get_default_value (param);
+		attrs |= defv ? GDA_VALUE_ATTR_CAN_BE_DEFAULT : 0;
 		mask |= GDA_VALUE_ATTR_CAN_BE_DEFAULT;
 
-		toset = gda_parameter_get_not_null (param);
+		toset = gda_holder_get_not_null (param);
 		attrs |= toset ? 0 : GDA_VALUE_ATTR_CAN_BE_NULL;
 		mask |= GDA_VALUE_ATTR_CAN_BE_NULL;
 
@@ -369,8 +368,8 @@ gnome_db_basic_form_dispose (GObject *object)
 
 	if (form->priv) {
 		/* paramlist */
-		if (form->priv->paramlist) 
-			paramlist_destroyed_cb (form->priv->paramlist, form);
+		if (form->priv->set) 
+			get_rid_of_set (form->priv->set, form);
 
 		gnome_db_basic_form_clean (form);
 
@@ -441,34 +440,66 @@ gnome_db_basic_form_set_property (GObject *object,
 #endif
 			break;
 		case PROP_PARAMLIST:
-			if (form->priv->paramlist) {
-				paramlist_destroyed_cb (form->priv->paramlist, form);
+			if (form->priv->set) {
+#ifdef HAVE_LIBGLADE
+			new_spec = NULL;
+			if (form->priv->layout_spec) {
+				/* old spec */
+				lspec = form->priv->layout_spec;
+				/* spec copy */
+				new_spec = g_new0 (GnomeDbFormLayoutSpec, 1);
+				if (lspec->xml_file)
+					new_spec->xml_file = g_strdup (lspec->xml_file);
+				if (lspec->xml_object) {
+					new_spec->xml_object = lspec->xml_object;
+					g_object_ref (new_spec->xml_object);
+				}
+				if (lspec->root_element)
+					new_spec->root_element = g_strdup (lspec->root_element);
+				if (lspec->form_prefix)
+					new_spec->form_prefix = g_strdup (lspec->form_prefix);
+			}
+#endif
+#ifdef HAVE_LIBGLADE
+			new_spec = NULL;
+			if (form->priv->layout_spec) {
+				/* old spec */
+				lspec = form->priv->layout_spec;
+				/* spec copy */
+				new_spec = g_new0 (GnomeDbFormLayoutSpec, 1);
+				if (lspec->xml_file)
+					new_spec->xml_file = g_strdup (lspec->xml_file);
+				if (lspec->xml_object) {
+					new_spec->xml_object = lspec->xml_object;
+					g_object_ref (new_spec->xml_object);
+				}
+				if (lspec->root_element)
+					new_spec->root_element = g_strdup (lspec->root_element);
+				if (lspec->form_prefix)
+					new_spec->form_prefix = g_strdup (lspec->form_prefix);
+			}
+#endif
+				get_rid_of_set (form->priv->set, form);
 				gnome_db_basic_form_clean (form);
 			}
 
-			form->priv->paramlist = g_value_get_pointer (value);
-			if (form->priv->paramlist) {
-				GError *error = NULL;
-				g_return_if_fail (GDA_IS_PARAMETER_LIST (form->priv->paramlist));
-				
-				if (! gda_parameter_list_is_coherent (form->priv->paramlist, &error)) {
-					g_warning ("gda_parameter_list_is_coherent() returned FALSE: %s", 
-						   error->message);
-					form->priv->paramlist = NULL;
-					g_error_free (error);
-					return;
-				}
-				
-				g_object_ref (form->priv->paramlist);
-				gda_object_connect_destroy (form->priv->paramlist,
-							    G_CALLBACK (paramlist_destroyed_cb), form);
-				g_signal_connect (form->priv->paramlist, "public_data_changed",
+			form->priv->set = g_value_get_pointer (value);
+			if (form->priv->set) {
+				g_return_if_fail (GDA_IS_SET (form->priv->set));
+								
+				g_object_ref (form->priv->set);
+				g_signal_connect (form->priv->set, "public_data_changed",
 						  G_CALLBACK (paramlist_public_data_changed_cb), form);
-				g_signal_connect (form->priv->paramlist, "param_plugin_changed",
+				g_signal_connect (form->priv->set, "holder_plugin_changed",
 						  G_CALLBACK (paramlist_param_plugin_changed_cb), form);
-				g_signal_connect (form->priv->paramlist, "param_attr_changed",
+				g_signal_connect (form->priv->set, "holder_attr_changed",
 						  G_CALLBACK (paramlist_param_attr_changed_cb), form);
-				
+
+#ifdef HAVE_LIBGLADE
+				if (new_spec)
+					form->priv->layout_spec = new_spec;
+				new_spec = NULL;
+#endif
 				gnome_db_basic_form_fill (form);
 			}
 			break;
@@ -500,7 +531,7 @@ gnome_db_basic_form_get_property (GObject *object,
         if (form->priv) {
                 switch (param_id) {
 		case PROP_PARAMLIST:
-			g_value_set_pointer (value, form->priv->paramlist);
+			g_value_set_pointer (value, form->priv->set);
 			break;
 		case PROP_HEADERS_SENSITIVE:
 			g_value_set_boolean (value, form->priv->headers_sensitive);
@@ -524,8 +555,8 @@ gnome_db_basic_form_clean (GnomeDbBasicForm *form)
 	GSList *list;
 	gint i = 0;
 
-	if (form->priv->paramlist) {
-		list = form->priv->paramlist->parameters;
+	if (form->priv->set) {
+		list = form->priv->set->holders;
 		while (list) {
 			g_signal_handler_disconnect (G_OBJECT (list->data), form->priv->signal_ids[i]);
 			
@@ -586,29 +617,26 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	GSList *list;
 	gint i;
 	gboolean form_expand = FALSE;
-	GdaDict *dict;
 	
 	/* parameters list management */
-	if (!form->priv->paramlist || !form->priv->paramlist->groups_list)
+	if (!form->priv->set || !form->priv->set->groups_list)
 		/* nothing to do */
 		return;
 
-	dict = gda_object_get_dict (GDA_OBJECT (form->priv->paramlist));
-
 	/* allocating space for the signal ids and connect to the parameter's changes */
-	form->priv->signal_ids = g_new0 (gulong, g_slist_length (form->priv->paramlist->parameters));
+	form->priv->signal_ids = g_new0 (gulong, g_slist_length (form->priv->set->holders));
 	i = 0;
 
 	/* creating all the data entries, and putting them into the form->priv->entries list */
-	list = form->priv->paramlist->groups_list;
+	list = form->priv->set->groups_list;
 	while (list) {
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 		GtkWidget *entry = NULL;
 
-		group = GDA_PARAMETER_LIST_GROUP (list->data);
+		group = GDA_SET_GROUP (list->data);
 		if (! group->nodes_source) { 
 			/* there is only one non-constrained parameter */
-			GdaParameter *param;
+			GdaHolder *param;
 			GType type;
 			const GValue *val, *default_val, *value;
 			gboolean nnul;
@@ -616,22 +644,22 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 
 			g_assert (g_slist_length (group->nodes) == 1);
 
-			param = GDA_PARAMETER (GDA_PARAMETER_LIST_NODE (group->nodes->data)->param);
+			param = GDA_HOLDER (GDA_SET_NODE (group->nodes->data)->holder);
 
-			val = gda_parameter_get_value (param);
-			default_val = gda_parameter_get_default_value (param);
-			nnul = gda_parameter_get_not_null (param);
+			val = gda_holder_get_value (param);
+			default_val = gda_holder_get_default_value (param);
+			nnul = gda_holder_get_not_null (param);
 
 			/* determine initial value */
-			type = gda_parameter_get_g_type (param);
+			type = gda_holder_get_g_type (param);
 			value = val;
 			if (!value && default_val && 
 			    (G_VALUE_TYPE ((GValue *) default_val) == type))
 				value = default_val;
 			
 			/* create entry */
-			g_object_get (G_OBJECT (param), "entry_plugin", &plugin, NULL);
-			entry = GTK_WIDGET (gnome_db_util_new_data_entry (dict, type, plugin));
+			g_object_get (G_OBJECT (param), "plugin", &plugin, NULL);
+			entry = GTK_WIDGET (gnome_db_util_new_data_entry (type, plugin));
 			g_free(plugin);
 
 
@@ -670,7 +698,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			GSList *plist;
 			gboolean nnul = TRUE;
 
-			entry = gnome_db_entry_combo_new (form->priv->paramlist, group->nodes_source);
+			entry = gnome_db_entry_combo_new (form->priv->set, group->nodes_source);
 			g_object_set_data (G_OBJECT (entry), "group", group);
 			g_object_set_data (G_OBJECT (entry), "form", form);
 			form->priv->entries = g_slist_append (form->priv->entries, entry);
@@ -679,10 +707,10 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			/* connect to the parameter's changes */
 			plist = group->nodes;
 			while (plist) {
-				GdaParameter *param;
+				GdaHolder *param;
 
-				param = GDA_PARAMETER_LIST_NODE (plist->data)->param;
-				if (!gda_parameter_get_not_null (param))
+				param = GDA_SET_NODE (plist->data)->holder;
+				if (!gda_holder_get_not_null (param))
 					nnul = FALSE;
 				form->priv->signal_ids[i] = g_signal_connect (param, "changed",
                                                                               G_CALLBACK (parameter_changed_cb), 
@@ -725,13 +753,13 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 			
 			gtk_box_pack_start (GTK_BOX (form), layout,  TRUE, TRUE, 0);
 			list = form->priv->entries;
-			groups = form->priv->paramlist->groups_list;
+			groups = form->priv->set->groups_list;
 			while (groups && list) {
 				gint param_no;
 				gchar *box_name;
 
-				param_no = g_slist_index (form->priv->paramlist->parameters,
-							  ((GdaParameterListNode *)(((GdaParameterListGroup *)groups->data)->nodes->data))->param);
+				param_no = g_slist_index (form->priv->set->holders,
+							  ((GdaSetNode *)(((GdaSetGroup *)groups->data)->nodes->data))->holder);
 				box_name = g_strdup_printf ("%s_%d", form->priv->layout_spec->form_prefix, param_no);
 				box = glade_xml_get_widget (form->priv->layout_spec->xml_object, box_name);
 				g_print ("Box named %s => %p\n", box_name, box);
@@ -774,26 +802,31 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 		while (list) {
 			gboolean expand;
 			GtkWidget *entry_label;
-			GdaParameter *param;
+			GdaHolder *param;
 			
 			/* label for the entry */
 			param = g_object_get_data (G_OBJECT (list->data), "param");
 			if (param) {
-				const gchar *title;
+				gchar *title;
 				gchar *str;
 				GtkWidget *evbox;
 
-				title = gda_object_get_name (GDA_OBJECT (param));
+				g_object_get (G_OBJECT (param), "name", &title, NULL);
 				if (!title)
-					title = _("Value");
+					title = g_strdup (_("Value"));
 				evbox = gtk_event_box_new ();
 				str = g_strdup_printf ("%s:", title);
 				label = gtk_label_new (str);
 				g_free (str);
-				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
-				if (gda_parameter_get_not_null (param)) 
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", title, g_free);
+				if (gda_holder_get_not_null (param)) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
-				
+
+#ifdef HAVE_LIBGLADE_FIXME
+				if (new_spec)
+					form->priv->layout_spec = new_spec;
+				new_spec = NULL;
+#endif
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
 				gtk_container_add (GTK_CONTAINER (evbox), label);
 				
@@ -803,38 +836,42 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				title = gda_object_get_description (GDA_OBJECT (param));
+				g_object_get (G_OBJECT (param), "description", &title, NULL);
 				if (title && *title)
 					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
+				g_free (title);
 			}
 			else {
 				/* FIXME: find a better label and tooltip and improve data entry attributes */
-				const gchar *title = NULL;
+				gchar *title = NULL;
 				gchar *str;
 				GtkWidget *evbox;
 				gboolean nullok = TRUE;
 				GSList *params;
-				GdaParameterListGroup *group;
+				GdaSetGroup *group;
 
 				group = g_object_get_data (G_OBJECT (list->data), "group");
-				params = group->nodes;
-				while (params) {
-					if (nullok && gda_parameter_get_not_null (GDA_PARAMETER_LIST_NODE (params->data)->param))
+				for (params = group->nodes; params; params = params->next) {
+					if (nullok && gda_holder_get_not_null (GDA_SET_NODE (params->data)->holder))
 						nullok = FALSE;
 					if (!title)
-						title = gda_object_get_name (GDA_OBJECT (GDA_PARAMETER_LIST_NODE (params->data)->param));
-					params = g_slist_next (params);
+						g_object_get (G_OBJECT (GDA_SET_NODE (params->data)->holder), 
+							      "name", &title, NULL);
 				}
 				
+				if (!title) {
+					str = g_object_get_data (G_OBJECT (group->nodes_source->data_model), 
+								 "name");
+					if (str)
+						title = g_strdup (str);
+				}
 				if (!title)
-					title = gda_object_get_name (GDA_OBJECT (group->nodes_source->data_model));
-				if (!title)
-					title = _("Value");
+					title = g_strdup (_("Value"));
 				evbox = gtk_event_box_new ();
 				str = g_strdup_printf ("%s:", title);
 				label = gtk_label_new (str);
 				g_free (str);
-				g_object_set_data_full (G_OBJECT (label), "_gda_title", g_strdup (title), g_free);
+				g_object_set_data_full (G_OBJECT (label), "_gda_title", title, g_free);
 				if (!nullok) 
 					form->priv->not_null_labels = g_slist_prepend (form->priv->not_null_labels, label);
 				gtk_misc_set_alignment (GTK_MISC (label), 0, 0.5);
@@ -846,7 +883,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 				gtk_widget_show (label);
 				entry_label = evbox;
 				
-				title = gda_object_get_description (GDA_OBJECT (group->nodes_source->data_model));
+				title = g_object_get_data (G_OBJECT (group->nodes_source->data_model), "descr");
 				if (title && *title)
 					gtk_tooltips_set_tip (form->priv->tooltips, evbox, title, NULL);
 			}
@@ -881,7 +918,7 @@ gnome_db_basic_form_fill (GnomeDbBasicForm *form)
 	/* hide some entries if specified */
 	for (list = form->priv->entries; list; list = list->next) {
 		gboolean explicit_hide = FALSE;
-		GdaParameter *param;
+		GdaHolder *param;
 			
 		param = g_object_get_data (G_OBJECT (list->data), "param");
 		if (param) {
@@ -963,7 +1000,7 @@ entry_contents_activated (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 static void
 entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 {
-	GdaParameter *param;
+	GdaHolder *param;
 	GdaValueAttribute attr;
 
 	attr = gnome_db_data_entry_get_attributes (entry);
@@ -971,16 +1008,15 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 	if (param) { /* single parameter */
 		GValue *value;
 		
-		form->priv->forward_param_updates = FALSE;
+		form->priv->forward_param_updates = TRUE;
 
 		/* parameter's value */
 		value = gnome_db_data_entry_get_value (entry);
 		if ((!value || gda_value_is_null (value)) &&
 		    (attr & GDA_VALUE_ATTR_IS_DEFAULT))
-			g_object_set (G_OBJECT (param), "use_default_value", TRUE, NULL);
+			gda_holder_set_value_to_default (param);
 		else
-			g_object_set (G_OBJECT (param), "use_default_value", FALSE, NULL);
-		gda_parameter_set_value (param, value);
+			gda_holder_set_value (param, value);
 #ifdef debug_signal
 		g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
@@ -988,13 +1024,12 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 #ifdef debug_signal
 		g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
-		form->priv->forward_param_updates = TRUE;
 		gda_value_free (value);
 	}
 	else { /* multiple parameters */
 		GSList *params;
 		GSList *values, *list;
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 
 		group = g_object_get_data (G_OBJECT (entry), "group");
 		params = group->nodes;
@@ -1012,12 +1047,12 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 			 * - modify the signal to add a list of parameters which are changed 
 			 *   and emit only one signal.
 			 */
-			GdaParameter *param;
+			GdaHolder *param;
 			form->priv->forward_param_updates = FALSE;
 
 			/* parameter's value */
-			param = GDA_PARAMETER_LIST_NODE (params->data)->param;
-			gda_parameter_set_value (param, (GValue *)(list->data));
+			param = GDA_SET_NODE (params->data)->holder;
+			gda_holder_set_value (param, (GValue *)(list->data));
 #ifdef debug_signal
 			g_print (">> 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
@@ -1026,7 +1061,7 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 #ifdef debug_signal
 			g_print ("<< 'PARAM_CHANGED' from %s\n", __FUNCTION__);
 #endif
-			form->priv->forward_param_updates = TRUE;;
+			form->priv->forward_param_updates = TRUE;
 
 			list = g_slist_next (list);
 			params = g_slist_next (params);
@@ -1035,13 +1070,13 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
 
 #ifdef PROXY_STORE_EXTRA_VALUES
 		/* updating the GdaDataProxy if there is one */
-		if (GDA_IS_DATA_MODEL_ITER (form->priv->paramlist)) {
+		if (GDA_IS_DATA_MODEL_ITER (form->priv->set)) {
 			GdaDataProxy *proxy;
 			gint proxy_row;
 			
-			proxy_row = gda_data_model_iter_get_row ((GdaDataModelIter *) form->priv->paramlist);
+			proxy_row = gda_data_model_iter_get_row ((GdaDataModelIter *) form->priv->set);
 
-			g_object_get (G_OBJECT (form->priv->paramlist), "data_model", &proxy, NULL);
+			g_object_get (G_OBJECT (form->priv->set), "data_model", &proxy, NULL);
 			if (GDA_IS_DATA_PROXY (proxy)) {
 				GSList *all_new_values;
 				gint i, col;
@@ -1071,11 +1106,11 @@ entry_contents_modified (GnomeDbDataEntry *entry, GnomeDbBasicForm *form)
  * the param change does not come from a GnomeDbDataEntry change.
  */ 
 static void
-parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
+parameter_changed_cb (GdaHolder *param, GnomeDbDataEntry *entry)
 {
 	GnomeDbBasicForm *form = g_object_get_data (G_OBJECT (entry), "form");
-	GdaParameterListGroup *group = g_object_get_data (G_OBJECT (entry), "group");
-	const GValue *value = gda_parameter_get_value (param);
+	GdaSetGroup *group = g_object_get_data (G_OBJECT (entry), "group");
+	const GValue *value = gda_holder_get_value (param);
 
 	if (form->priv->forward_param_updates) {
 		gboolean param_valid;
@@ -1083,7 +1118,7 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 
 		/* There can be a feedback from the entry if the param is invalid and "set_default_if_invalid"
 		   exists and is TRUE */
-		param_valid = gda_parameter_is_valid (param);
+		param_valid = gda_holder_is_valid (param);
 		if (!param_valid) 
 			if (g_object_class_find_property (G_OBJECT_GET_CLASS (entry), "set_default_if_invalid"))
 				g_object_get (G_OBJECT (entry), "set_default_if_invalid", &default_if_invalid, NULL);
@@ -1102,7 +1137,7 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
 
 			while (list) {
 				const GValue *pvalue;
-				pvalue = gda_parameter_get_value (GDA_PARAMETER_LIST_NODE (list->data)->param);
+				pvalue = gda_holder_get_value (GDA_SET_NODE (list->data)->holder);
 				values = g_slist_append (values, (GValue *) pvalue);
 				if (allnull && pvalue && 
 				    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
@@ -1142,18 +1177,18 @@ parameter_changed_cb (GdaParameter *param, GnomeDbDataEntry *entry)
  * gnome_db_basic_form_get_paramlist
  * @form: a #GnomeDbBasicForm widget
  *
- * Get a pointer to the #GdaParameterList used internally by @form to store
+ * Get a pointer to the #GdaSet used internally by @form to store
  * values
  *
  * Returns:
  */
-GdaParameterList *
+GdaSet *
 gnome_db_basic_form_get_paramlist (GnomeDbBasicForm *form)
 {
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
-	return form->priv->paramlist;
+	return form->priv->set;
 }
 
 /**
@@ -1169,14 +1204,14 @@ void
 gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 {
 	GSList *list;
-	GdaParameter *param;
+	GdaHolder *param;
 
 	g_return_if_fail (form && GNOME_DB_IS_BASIC_FORM (form));
 	g_return_if_fail (form->priv);
 
 	list = form->priv->entries;
 	while (list) {
-		GdaParameterListGroup *group;
+		GdaSetGroup *group;
 
 		group = g_object_get_data (G_OBJECT (list->data), "group");
 
@@ -1188,7 +1223,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 			
 			while (params) {
 				const GValue *pvalue;
-				pvalue = gda_parameter_get_value (GDA_PARAMETER_LIST_NODE (params->data)->param);
+				pvalue = gda_holder_get_value (GDA_SET_NODE (params->data)->holder);
 				values = g_slist_append (values, (GValue *) pvalue);
 				if (allnull && pvalue && 
 				    (G_VALUE_TYPE ((GValue *) pvalue) != GDA_TYPE_NULL))
@@ -1207,7 +1242,7 @@ gnome_db_basic_form_set_current_as_orig (GnomeDbBasicForm *form)
 		else {
 			/* non combo entry */
 			param = g_object_get_data (G_OBJECT (list->data), "param");
-			gnome_db_data_entry_set_value_orig (GNOME_DB_DATA_ENTRY (list->data), gda_parameter_get_value (param));
+			gnome_db_data_entry_set_value_orig (GNOME_DB_DATA_ENTRY (list->data), gda_holder_get_value (param));
 		}
 		list = g_slist_next (list);
 	}
@@ -1227,25 +1262,25 @@ gnome_db_basic_form_is_valid (GnomeDbBasicForm *form)
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), FALSE);
 	g_return_val_if_fail (form->priv, FALSE);
 
-	return gda_parameter_list_is_valid (form->priv->paramlist);
+	return gda_set_is_valid (form->priv->set);
 }
 
 /**
  * gnome_db_basic_form_get_data_set
  * @form: a #GnomeDbBasicForm widget
  *
- * Get a pointer to the #GdaParameterList object which
+ * Get a pointer to the #GdaSet object which
  * is modified by @form
  *
  * Returns:
  */
-GdaParameterList *
+GdaSet *
 gnome_db_basic_form_get_data_set (GnomeDbBasicForm *form)
 {
 	g_return_val_if_fail (form && GNOME_DB_IS_BASIC_FORM (form), NULL);
 	g_return_val_if_fail (form->priv, NULL);
 
-	return form->priv->paramlist;
+	return form->priv->set;
 }
 
 /**
@@ -1322,7 +1357,7 @@ gnome_db_basic_form_reset (GnomeDbBasicForm *form)
 	
 	list = form->priv->entries;
 	while (list) {
-		GdaParameterListNode *node = g_object_get_data (G_OBJECT (list->data), "node");
+		GdaSetNode *node = g_object_get_data (G_OBJECT (list->data), "node");
 
 		if (node) {
 			/* Combo entry */
@@ -1347,14 +1382,14 @@ gnome_db_basic_form_reset (GnomeDbBasicForm *form)
 /**
  * gnome_db_basic_form_entry_show
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * @show:
  *
  * Shows or hides the #GnomeDbDataEntry in @form which corresponds to the
  * @param parameter
  */
 void
-gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gboolean show)
+gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaHolder *param, gboolean show)
 {
 	GtkWidget *entry;
 
@@ -1416,12 +1451,12 @@ gnome_db_basic_form_entry_show (GnomeDbBasicForm *form, GdaParameter *param, gbo
 /**
  * gnome_db_basic_form_entry_grab_focus
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * 
  * Makes the data entry corresponding to @param grab the focus for the window it's in
  */
 void
-gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GtkWidget *entry;
 
@@ -1437,14 +1472,14 @@ gnome_db_basic_form_entry_grab_focus (GnomeDbBasicForm *form, GdaParameter *para
 /**
  * gnome_db_basic_form_entry_set_editable
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  * @editable:
  *
  * Sets the #GnomeDbDataEntry in @form which corresponds to the
  * @param parameter editable or not
  */
 void
-gnome_db_basic_form_entry_set_editable (GnomeDbBasicForm *form, GdaParameter *param, gboolean editable)
+gnome_db_basic_form_entry_set_editable (GnomeDbBasicForm *form, GdaHolder *param, gboolean editable)
 {
 	GtkWidget *entry;
 
@@ -1516,19 +1551,19 @@ gnome_db_basic_form_set_entries_default (GnomeDbBasicForm *form)
 	}
 }
 
-static void form_param_changed (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkDialog *dlg);
+static void form_param_changed (GnomeDbBasicForm *form, GdaHolder *param, gboolean is_user_modif, GtkDialog *dlg);
 
 /**
  * gnome_db_basic_form_get_entry_widget
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  *
  * Get the #GnomeDbDataEntry in @form which corresponds to the param parameter.
  *
  * Returns: the requested widget, or %NULL if not found
  */
 GtkWidget *
-gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GSList *entries;
 	GtkWidget *entry = NULL;
@@ -1538,7 +1573,7 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 
 	entries = form->priv->entries;
 	while (entries && !entry) {
-		GdaParameter *thisparam = g_object_get_data (G_OBJECT (entries->data), "param");
+		GdaHolder *thisparam = g_object_get_data (G_OBJECT (entries->data), "param");
 
 		if (thisparam) {
 			if (thisparam == param)
@@ -1547,12 +1582,12 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 		else {
 			/* multiple parameters */
 			GSList *params;
-			GdaParameterListGroup *group;
+			GdaSetGroup *group;
 
 			group = g_object_get_data (G_OBJECT (entries->data), "group");
 			params = group->nodes;
 			while (params && !entry) {
-				if (GDA_PARAMETER_LIST_NODE (params->data)->param == (gpointer) param)
+				if (GDA_SET_NODE (params->data)->holder == (gpointer) param)
 					entry = GTK_WIDGET (entries->data);
 				params = g_slist_next (params);
 			}
@@ -1567,14 +1602,14 @@ gnome_db_basic_form_get_entry_widget (GnomeDbBasicForm *form, GdaParameter *para
 /**
  * gnome_db_basic_form_get_label_widget
  * @form: a #GnomeDbBasicForm widget
- * @param: a #GdaParameter object
+ * @param: a #GdaHolder object
  *
  * Get the label in @form which corresponds to the param parameter.
  *
  * Returns: the requested widget, or %NULL if not found
  */
 GtkWidget *
-gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *param)
+gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaHolder *param)
 {
 	GtkWidget *entry;
 
@@ -1591,8 +1626,7 @@ gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *para
 
 /**
  * gnome_db_basic_form_new_in_dialog
- * @dict: a #GdaDict object
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  * @parent: the parent window for the new dialog, or %NULL
  * @title: the title of the dialog window, or %NULL
  * @header: a helper text displayed at the top of the dialog, or %NULL
@@ -1607,7 +1641,7 @@ gnome_db_basic_form_get_label_widget (GnomeDbBasicForm *form, GdaParameter *para
  * Returns: the new #GtkDialog widget
  */
 GtkWidget *
-gnome_db_basic_form_new_in_dialog (GdaParameterList *paramlist, GtkWindow *parent,
+gnome_db_basic_form_new_in_dialog (GdaSet *paramlist, GtkWindow *parent,
 				   const gchar *title, const gchar *header)
 {
 	GtkWidget *form;
@@ -1651,7 +1685,7 @@ gnome_db_basic_form_new_in_dialog (GdaParameterList *paramlist, GtkWindow *paren
 }
 
 static void
-form_param_changed (GnomeDbBasicForm *form, GdaParameter *param, gboolean is_user_modif, GtkDialog *dlg)
+form_param_changed (GnomeDbBasicForm *form, GdaHolder *param, gboolean is_user_modif, GtkDialog *dlg)
 {
 	gboolean valid;
 
