@@ -1,6 +1,6 @@
 /* gnome-db-data-store.c
  *
- * Copyright (C) 2005 - 2006 Vivien Malerba
+ * Copyright (C) 2005 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -47,6 +47,7 @@ enum
 
 struct _GnomeDbDataStorePriv {
 	GdaDataProxy *proxy;
+	gint          nrows;
 	gint          stamp; /* Random integer to check whether an iter belongs to our model */
 };
 
@@ -149,6 +150,7 @@ gnome_db_data_store_init (GnomeDbDataStore *store)
 	/* Private structure */
 	store->priv = g_new0 (GnomeDbDataStorePriv, 1);
 	store->priv->proxy = NULL;
+	store->priv->nrows = 0;
 	store->priv->stamp = g_random_int ();
 }
 
@@ -158,6 +160,7 @@ row_inserted_cb (GdaDataProxy *proxy, gint row, GtkTreeModel *store)
 	GtkTreePath *path;
 	GtkTreeIter iter;
 
+	((GnomeDbDataStore*) store)->priv->nrows ++;
 	((GnomeDbDataStore*) store)->priv->stamp = g_random_int ();
 	path = gtk_tree_path_new ();
         gtk_tree_path_append_index (path, row);
@@ -185,6 +188,7 @@ row_removed_cb (GdaDataProxy *proxy, gint row, GtkTreeModel *store)
 {
 	GtkTreePath *path;
 
+	((GnomeDbDataStore*) store)->priv->nrows --;
 	((GnomeDbDataStore*) store)->priv->stamp = g_random_int ();
 	path = gtk_tree_path_new ();
         gtk_tree_path_append_index (path, row);
@@ -193,19 +197,17 @@ row_removed_cb (GdaDataProxy *proxy, gint row, GtkTreeModel *store)
 }
 
 static void
-proxy_destroyed_cb (GdaDataProxy *proxy, GnomeDbDataStore *store)
+proxy_reset_cb (GdaDataProxy *proxy, GtkTreeModel *store)
 {
-	g_signal_handlers_disconnect_by_func (proxy,
-					      G_CALLBACK (proxy_destroyed_cb), store);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
-					      G_CALLBACK (row_inserted_cb), store);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
-					      G_CALLBACK (row_updated_cb), store);
-	g_signal_handlers_disconnect_by_func (G_OBJECT (proxy),
-					      G_CALLBACK (row_removed_cb), store);
-	g_object_unref (store->priv->proxy);
-	store->priv->proxy = NULL;
-	store->priv->stamp = g_random_int ();
+	gint i, nrows;
+
+	while (((GnomeDbDataStore*) store)->priv->nrows > 0) 
+		row_removed_cb (proxy, ((GnomeDbDataStore*) store)->priv->nrows - 1, store);
+
+	nrows = gda_data_model_get_n_rows ((GdaDataModel *) proxy);;
+	((GnomeDbDataStore*) store)->priv->nrows = 0;
+	for (i = 0; i < nrows; i++) 
+		row_inserted_cb (proxy, i, store);
 }
 
 static void
@@ -219,8 +221,21 @@ gnome_db_data_store_dispose (GObject *object)
 	store = GNOME_DB_DATA_STORE (object);
 
 	if (store->priv) {
-		if (store->priv->proxy)
-			proxy_destroyed_cb (store->priv->proxy, store);
+		if (store->priv->proxy) {
+			g_signal_handlers_disconnect_by_func (G_OBJECT (store->priv->proxy),
+							      G_CALLBACK (row_inserted_cb), store);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (store->priv->proxy),
+							      G_CALLBACK (row_updated_cb), store);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (store->priv->proxy),
+							      G_CALLBACK (row_removed_cb), store);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (store->priv->proxy),
+							      G_CALLBACK (proxy_reset_cb), store);
+			g_object_unref (store->priv->proxy);
+			store->priv->proxy = NULL;
+			store->priv->nrows = 0;
+			store->priv->stamp = g_random_int ();
+		}
+
 		g_free (store->priv);
 		store->priv = NULL;
 	}
@@ -254,9 +269,9 @@ gnome_db_data_store_set_property (GObject *object,
 			else
 				proxy = (GdaDataProxy *) gda_data_proxy_new (model);
 			store->priv->proxy = proxy;
-			
-			gda_object_connect_destroy (store->priv->proxy,
-						    G_CALLBACK (proxy_destroyed_cb), store);
+			g_object_set (G_OBJECT (proxy), "defer_sync", FALSE, "sample_size", 0, NULL);
+			store->priv->nrows = gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy));
+
 			/* connect to row changes */
 			g_signal_connect (G_OBJECT (proxy), "row_inserted",
 					  G_CALLBACK (row_inserted_cb), store);
@@ -264,7 +279,10 @@ gnome_db_data_store_set_property (GObject *object,
 					  G_CALLBACK (row_updated_cb), store);
 			g_signal_connect (G_OBJECT (proxy), "row_removed",
 					  G_CALLBACK (row_removed_cb), store);
+			g_signal_connect (G_OBJECT (proxy), "reset",
+					  G_CALLBACK (proxy_reset_cb), store);
 			store->priv->stamp = g_random_int ();
+			proxy_reset_cb (GDA_DATA_PROXY (proxy), (GtkTreeModel *) store);
 			break;
 		case PROP_ADD_NULL_ENTRY:
 			g_return_if_fail (store->priv->proxy);
@@ -298,7 +316,6 @@ gnome_db_data_store_get_property (GObject *object,
 		case PROP_ADD_NULL_ENTRY: {
 			gboolean prop;
 			
-			g_return_if_fail (store->priv->proxy);
 			g_object_get (store->priv->proxy, "prepend_null_entry", &prop, NULL);
 			g_value_set_boolean (value, prop);
 			break;
@@ -388,6 +405,7 @@ gnome_db_data_store_set_value (GnomeDbDataStore *store, GtkTreeIter *iter,
 						       g_value_get_uint (value));
 		return TRUE;
 	}
+	return FALSE;
 }
 
 /**
@@ -630,7 +648,7 @@ data_store_get_iter (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreePath *p
         g_return_val_if_fail (depth == 1, FALSE);
 
         n = indices[0]; /* the n-th top level row */
-        if (n < gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy))) {
+        if (n < store->priv->nrows) {
                 iter->stamp = store->priv->stamp;
                 iter->user_data = GINT_TO_POINTER (n);
                 return TRUE;
@@ -757,7 +775,7 @@ data_store_iter_next (GtkTreeModel *tree_model, GtkTreeIter *iter)
 
         row = GPOINTER_TO_INT (iter->user_data);
         row++;
-        if (row >= gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy)))
+        if (row >= store->priv->nrows)
                 return FALSE;
         else {
                 iter->user_data = GINT_TO_POINTER (row);
@@ -776,7 +794,7 @@ data_store_iter_children (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeIt
         g_return_val_if_fail (store->priv->proxy, FALSE);
         g_return_val_if_fail (iter, FALSE);
 
-        if (!parent && (gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy)) > 0)) {
+        if (!parent && (store->priv->nrows > 0)) {
                 iter->stamp = store->priv->stamp;
                 iter->user_data = GINT_TO_POINTER (0);
                 return TRUE;
@@ -802,7 +820,7 @@ data_store_iter_n_children (GtkTreeModel *tree_model, GtkTreeIter *iter)
         g_return_val_if_fail (store->priv->proxy, -1);
 
         if (!iter)
-                return gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy));
+                return store->priv->nrows;
         else
                 return 0;
 }
@@ -818,7 +836,7 @@ data_store_iter_nth_child (GtkTreeModel *tree_model, GtkTreeIter *iter, GtkTreeI
         g_return_val_if_fail (store->priv->proxy, FALSE);
         g_return_val_if_fail (iter, FALSE);
 
-        if (!parent && (n < gda_data_model_get_n_rows (GDA_DATA_MODEL (store->priv->proxy)))) {
+        if (!parent && (n < store->priv->nrows)) {
                 iter->stamp = store->priv->stamp;
                 iter->user_data = GINT_TO_POINTER (n);
                 return TRUE;
