@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2007 The GNOME Foundation
+ * Copyright (C) 1999 - 2008 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -22,13 +22,13 @@
  */
 
 #include <string.h>
-#include <libgda/gda-client.h>
 #include <libgda/gda-config.h>
-#include <libgda/gda-parameter-list.h>
-#include <libgda/gda-parameter.h>
+#include <libgda/gda-set.h>
+#include <libgda/gda-holder.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <libgnomedb/gnome-db-dsn-assistant.h>
-#include <libgnomedb/gnome-db-dsn-spec.h>
+#include <libgnomedb/gnome-db-provider-spec-editor.h>
+#include <libgnomedb/gnome-db-provider-auth-editor.h>
 #include <libgnomedb/gnome-db-provider-selector.h>
 #include <libgnomedb/gnome-db-util.h>
 #include <libgnomedb/gnome-db-server-operation.h>
@@ -44,11 +44,11 @@ enum {
 	PAGE_OPT_CREATE_DB   = 2,
 	PAGE_CREATE_DB_INFO  = 3,
 	PAGE_CONNECT_INFO    = 4,
-	PAGE_LAST            = 5
+	PAGE_AUTH_INFO       = 5,
+	PAGE_LAST            = 6
 };
 
 struct _GnomeDbDsnAssistantPrivate {
-	GdaClient          *client;
 	GdaDataSourceInfo  *dsn_info;
 	GdaServerOperation *create_db_op;
 
@@ -59,9 +59,7 @@ struct _GnomeDbDsnAssistantPrivate {
 	GtkWidget *general_name;
 	GtkWidget *general_provider;
 	GtkWidget *general_description;
-	GtkWidget *general_username;
-	GtkWidget *general_password;
-	GtkWidget *general_is_global;
+	GtkWidget *general_is_system;
 
 	GtkWidget *choose_toggle;
 
@@ -71,6 +69,10 @@ struct _GnomeDbDsnAssistantPrivate {
 	GtkWidget *cnc_params_page;
 	GtkWidget *provider_container;
 	GtkWidget *provider_detail;
+
+	GtkWidget *cnc_auth_page;
+	GtkWidget *auth_container;
+	GtkWidget *auth_detail;
 };
 
 static void gnome_db_dsn_assistant_class_init (GnomeDbDsnAssistantClass *klass);
@@ -86,6 +88,16 @@ enum {
 static guint config_assistant_signals[LAST_SIGNAL] = { 0, };
 static GObjectClass *parent_class = NULL;
 
+static void 
+data_source_info_free (GdaDataSourceInfo *info)
+{
+	g_free (info->provider); 
+	g_free (info->cnc_string); 
+	g_free (info->description);
+	g_free (info->auth_string);
+	g_free (info);
+}
+
 /*
  * Callbacks
  */
@@ -109,7 +121,7 @@ assistant_applied_cb (GtkAssistant *assist, gpointer data)
 
 	/* clear the internal dsn_info */
 	if (assistant->priv->dsn_info) {
-		gda_data_source_info_free (assistant->priv->dsn_info);
+		data_source_info_free (assistant->priv->dsn_info);
 		assistant->priv->dsn_info = NULL;
 	}
 
@@ -125,8 +137,7 @@ assistant_applied_cb (GtkAssistant *assist, gpointer data)
 			GSList *dsn_params;
 			GError *error = NULL;
 
-			allok = gda_client_perform_create_database (assistant->priv->client, 
-								    assistant->priv->create_db_op, &error);
+			allok = gda_perform_create_database (assistant->priv->create_db_op, &error);
 			if (!allok) {
 				gchar *str;
 				str = g_strdup_printf (_("Error creating database: %s"), 
@@ -139,37 +150,32 @@ assistant_applied_cb (GtkAssistant *assist, gpointer data)
 			}
 			
 			/* make the connection string for the data source */
-			prov_info = gda_config_get_provider_by_name (
-                                             gnome_db_provider_selector_get_selected_provider (
-					     GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider)));
+			prov_info = gda_config_get_provider_info (gnome_db_provider_selector_get_provider 
+								  (GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider)));
 			g_return_if_fail (prov_info);
-			dsn_params = prov_info->gda_params->parameters;
-			while (dsn_params) {
-				GdaParameter *param = GDA_PARAMETER (dsn_params->data);
+			for (dsn_params = prov_info->dsn_params->holders; dsn_params; dsn_params = dsn_params->next) {
+				GdaHolder *param = GDA_HOLDER (dsn_params->data);
 				const GValue *value;
 				
 				value = gda_server_operation_get_value_at (assistant->priv->create_db_op,
 									   "/DB_DEF_P/%s",
-									   gda_object_get_id (GDA_OBJECT (param)));
+									   gda_holder_get_id (param));
 				if (!value)
 					value = gda_server_operation_get_value_at (assistant->priv->create_db_op,
-									   "/SERVER_CNX_P/%s",
-									   gda_object_get_id (GDA_OBJECT (param)));
+										   "/SERVER_CNX_P/%s",
+										   gda_holder_get_id (param));
 				
 				if (value && !gda_value_is_null ((GValue *) value)) {
 					gchar *str;
 
-					if (dsn_params == prov_info->gda_params->parameters)
+					if (dsn_params == prov_info->dsn_params->holders)
 						cnc_string = g_string_new ("");
 					else
 						g_string_append (cnc_string, ";");
 					str = gda_value_stringify ((GValue *) value);
-					g_string_append_printf (cnc_string, "%s=%s",
-								gda_object_get_id (GDA_OBJECT (param)), str);
+					g_string_append_printf (cnc_string, "%s=%s", gda_holder_get_id (param), str);
 					g_free (str);
 				}
-				
-				dsn_params = g_slist_next (dsn_params);
 			}
 		}
 	}
@@ -179,28 +185,23 @@ assistant_applied_cb (GtkAssistant *assist, gpointer data)
 		assistant->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 		assistant->priv->dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (assistant->priv->general_name)));
 		assistant->priv->dsn_info->provider = g_strdup (
-					 gnome_db_provider_selector_get_selected_provider (
+					 gnome_db_provider_selector_get_provider (
 					 GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider)));
 		if (cnc_string) {
 			assistant->priv->dsn_info->cnc_string = cnc_string->str;
 			g_string_free (cnc_string, FALSE);
 		}
 		else
-			assistant->priv->dsn_info->cnc_string = gnome_db_dsn_spec_get_specs 
-				(GNOME_DB_DSN_SPEC (assistant->priv->provider_detail));
+			assistant->priv->dsn_info->cnc_string = gnome_db_provider_spec_editor_get_specs 
+				(GNOME_DB_PROVIDER_SPEC_EDITOR (assistant->priv->provider_detail));
 		assistant->priv->dsn_info->description =
 			g_strdup (gtk_entry_get_text (GTK_ENTRY (assistant->priv->general_description)));
-		assistant->priv->dsn_info->username =
-			g_strdup (gtk_entry_get_text (GTK_ENTRY (assistant->priv->general_username)));
-		assistant->priv->dsn_info->password =
-			g_strdup (gtk_entry_get_text (GTK_ENTRY (assistant->priv->general_password)));
-		if (gda_config_can_modify_global_config ())
-			assistant->priv->dsn_info->is_global = gtk_toggle_button_get_active 
-				(GTK_TOGGLE_BUTTON (assistant->priv->general_is_global));
+		assistant->priv->dsn_info->auth_string = NULL; TO_IMPLEMENT;
+		if (gda_config_can_modify_system_config ())
+			assistant->priv->dsn_info->is_system = gtk_toggle_button_get_active 
+				(GTK_TOGGLE_BUTTON (assistant->priv->general_is_system));
 		else
-			assistant->priv->dsn_info->is_global = FALSE;
-
-		allok = gda_config_save_data_source_info (assistant->priv->dsn_info);
+			assistant->priv->dsn_info->is_system = FALSE;
 	}
 
 	/* notify listeners */
@@ -210,25 +211,29 @@ assistant_applied_cb (GtkAssistant *assist, gpointer data)
 static GdaServerOperation *
 get_specs_database_creation (GnomeDbDsnAssistant *assistant)
 {
-	if (! assistant->priv->create_db_op) {
-		if (!assistant->priv->client)
-			assistant->priv->client = gda_client_new ();
-		
+	if (! assistant->priv->create_db_op) 
 		assistant->priv->create_db_op = 
-			gda_client_prepare_create_database (assistant->priv->client, NULL,
-					  gnome_db_provider_selector_get_selected_provider (
-					  GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider)));
-	}
+			gda_prepare_create_database (gnome_db_provider_selector_get_provider (
+						     GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider)),
+						     NULL, NULL);
 
 	return assistant->priv->create_db_op;
 }
 
 static void
-dsn_spec_changed_cb (GnomeDbDsnSpec *spec, GnomeDbDsnAssistant *assistant)
+dsn_spec_changed_cb (GnomeDbProviderSpecEditor *spec, GnomeDbDsnAssistant *assistant)
 {
 	gtk_assistant_set_page_complete (GTK_ASSISTANT (assistant),
 					 assistant->priv->cnc_params_page,
-					 gnome_db_dsn_spec_is_valid (spec));
+					 gnome_db_provider_spec_editor_is_valid (spec));
+}
+
+static void
+dsn_auth_changed_cb (GnomeDbProviderAuthEditor *auth, GnomeDbDsnAssistant *assistant)
+{
+	gtk_assistant_set_page_complete (GTK_ASSISTANT (assistant),
+					 assistant->priv->cnc_auth_page,
+					 gnome_db_provider_auth_editor_is_valid (auth));
 }
 
 static void
@@ -251,7 +256,6 @@ provider_changed_cb (GtkWidget *combo, GnomeDbDsnAssistant *assistant)
 	/* is the database creation supported by the chosen provider? */
 	op = get_specs_database_creation (assistant);
 	if (op) {
-		/*assistant->priv->newdb_params = gnome_db_server_operation_new (op);*/
 		assistant->priv->newdb_params = g_object_new (GNOME_DB_TYPE_SERVER_OPERATION, 
 							      "opt_header", TRUE, "server_operation", op, NULL);
 		gtk_widget_show (assistant->priv->newdb_params);
@@ -264,11 +268,10 @@ provider_changed_cb (GtkWidget *combo, GnomeDbDsnAssistant *assistant)
 		gtk_widget_set_sensitive (assistant->priv->choose_toggle, FALSE);
 
 	/* dsn spec for the selected provider */
-	provider = gnome_db_provider_selector_get_selected_provider (
-			     GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider));
+	provider = gnome_db_provider_selector_get_provider (GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider));
 	g_assert (provider);
 	if (!assistant->priv->provider_detail) {
-		assistant->priv->provider_detail = gnome_db_dsn_spec_new (provider);
+		assistant->priv->provider_detail = gnome_db_provider_spec_editor_new (provider);
 		gtk_box_pack_start (GTK_BOX (assistant->priv->provider_container),
 				    assistant->priv->provider_detail, TRUE, TRUE, 0);
 		gtk_widget_show (assistant->priv->provider_detail);
@@ -276,7 +279,19 @@ provider_changed_cb (GtkWidget *combo, GnomeDbDsnAssistant *assistant)
 				  G_CALLBACK (dsn_spec_changed_cb), assistant);
 	}
 	else
-		gnome_db_dsn_spec_set_provider (GNOME_DB_DSN_SPEC (assistant->priv->provider_detail), provider);
+		gnome_db_provider_spec_editor_set_provider (GNOME_DB_PROVIDER_SPEC_EDITOR (assistant->priv->provider_detail), provider);
+
+	/* dsn authentication for the selected provider */
+	if (!assistant->priv->auth_detail) {
+		assistant->priv->auth_detail = gnome_db_provider_auth_editor_new (provider);
+		gtk_box_pack_start (GTK_BOX (assistant->priv->auth_container),
+				    assistant->priv->auth_detail, TRUE, TRUE, 0);
+		gtk_widget_show (assistant->priv->auth_detail);
+		g_signal_connect (assistant->priv->auth_detail, "changed",
+				  G_CALLBACK (dsn_auth_changed_cb), assistant);
+	}
+	else
+		gnome_db_provider_auth_editor_set_provider (GNOME_DB_PROVIDER_AUTH_EDITOR (assistant->priv->auth_detail), provider);
 }
 
 static void
@@ -293,16 +308,15 @@ dsn_name_changed_cb (GtkEntry *entry, GnomeDbDsnAssistant *assistant)
 		page_complete = FALSE;
 	}
 
-	dsn_info = gda_config_find_data_source (name);
+	dsn_info = gda_config_get_dsn (name);
 	if (dsn_info) {
 		gint i = 2;
 		gchar *str = NULL;
 
 		do {
 			g_free (str);
-			gda_data_source_info_free (dsn_info);
 			str = g_strdup_printf ("%s_%d", name, i);
-			dsn_info = gda_config_find_data_source (str);
+			dsn_info = gda_config_get_dsn (str);
 		} while (dsn_info);
 
 		gtk_entry_set_text (entry, str);
@@ -357,7 +371,19 @@ forward_page_function (gint current_page, GnomeDbDsnAssistant *assistant)
 			return PAGE_CONNECT_INFO;
 	case PAGE_CREATE_DB_INFO:
 		return PAGE_LAST;
-	case PAGE_CONNECT_INFO:
+	case PAGE_CONNECT_INFO: {
+		GdaProviderInfo *pinfo;
+		const gchar *provider;
+		provider = gnome_db_provider_selector_get_provider (GNOME_DB_PROVIDER_SELECTOR (assistant->priv->general_provider));
+		g_assert (provider);
+		pinfo = gda_config_get_provider_info (provider);
+		g_assert (pinfo);
+		if (pinfo->auth_params && pinfo->auth_params->holders) 
+			return PAGE_AUTH_INFO;
+		else
+			return PAGE_LAST;
+	}
+	case PAGE_AUTH_INFO:
 		return PAGE_LAST;
 	case PAGE_LAST:
 		break;
@@ -417,7 +443,7 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 	 */
 	vbox = gtk_vbox_new (FALSE, 5);
 
-	table = gtk_table_new (7, 2, FALSE);
+	table = gtk_table_new (5, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox), table, FALSE, FALSE, 0);
 	gtk_container_set_border_width (GTK_CONTAINER (table), 10);
 	gtk_table_set_row_spacings (GTK_TABLE (table), 3);
@@ -445,16 +471,16 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 	g_signal_connect (assistant->priv->general_name, "changed",
 			  G_CALLBACK (dsn_name_changed_cb), assistant);
 
-	if (gda_config_can_modify_global_config ()) {
+	if (gda_config_can_modify_system_config ()) {
 		label = gtk_label_new (_("System wide data source:"));
 		gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
 		gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, 0, 0, 0);
 
-		assistant->priv->general_is_global = gtk_check_button_new ();
-		gtk_table_attach_defaults (GTK_TABLE (table), assistant->priv->general_is_global, 1, 2, 2, 3);
+		assistant->priv->general_is_system = gtk_check_button_new ();
+		gtk_table_attach_defaults (GTK_TABLE (table), assistant->priv->general_is_system, 1, 2, 2, 3);
 	}
 	else
-		assistant->priv->general_is_global = NULL;
+		assistant->priv->general_is_system = NULL;
 
 	str = gnome_db_utility_markup_title (_("Database type"), FALSE);
 	label = gtk_label_new ("");
@@ -474,20 +500,6 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 	
 	assistant->priv->general_description = gnome_db_new_entry_widget (0, TRUE);
 	gtk_table_attach_defaults (GTK_TABLE (table), assistant->priv->general_description, 1, 2, 4, 5);
-
-	label = gtk_label_new (_("Username:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, 0, 0, 0);
-	
-	assistant->priv->general_username = gnome_db_new_entry_widget (0, TRUE);
-	gtk_table_attach_defaults (GTK_TABLE (table), assistant->priv->general_username, 1, 2, 5, 6);
-
-	label = gtk_label_new (_("Password:"));
-	gtk_misc_set_alignment (GTK_MISC (label), 0., 0.);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 6, 7, GTK_FILL, 0, 0, 0);
-	
-	assistant->priv->general_password = gnome_db_new_entry_widget (0, TRUE);
-	gtk_table_attach_defaults (GTK_TABLE (table), assistant->priv->general_password, 1, 2, 6, 7);
 	
 	gtk_widget_show_all (vbox);
 
@@ -592,6 +604,30 @@ gnome_db_dsn_assistant_init (GnomeDbDsnAssistant *assistant,
 	gtk_assistant_set_page_type (assist, vbox, GTK_ASSISTANT_PAGE_CONTENT);
 	assistant->priv->cnc_params_page = vbox;
 
+	/* 
+	 * authentication page 
+	 */
+	vbox = gtk_vbox_new (FALSE, 5);
+	gtk_container_set_border_width (GTK_CONTAINER (vbox), 10);
+	
+	label = gnome_db_new_label_widget (NULL);
+	gtk_label_set_markup (GTK_LABEL (label),
+			      _("The following fields represent the authentication information needed "
+				"to open a connection."));
+	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	gtk_box_pack_start (GTK_BOX (vbox), label, FALSE, FALSE, 0);
+
+	assistant->priv->auth_container = gnome_db_new_vbox_widget (FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (vbox), assistant->priv->auth_container, TRUE, TRUE, 0);
+
+	gtk_widget_show_all (vbox);
+
+	gtk_assistant_append_page (assist, vbox);
+	gtk_assistant_set_page_title (assist, vbox, _("Authentication parameters"));
+	gtk_assistant_set_page_header_image (assist, vbox, assistant->priv->logo);
+	gtk_assistant_set_page_type (assist, vbox, GTK_ASSISTANT_PAGE_CONTENT);
+	assistant->priv->cnc_auth_page = vbox;
+
 	/* 
 	 * end page 
 	 */
@@ -626,13 +662,10 @@ gnome_db_dsn_assistant_finalize (GObject *object)
 	g_return_if_fail (GNOME_DB_IS_DSN_ASSISTANT (assistant));
 
 	/* free memory */
-	if (assistant->priv->client)
-		g_object_unref (assistant->priv->client);
-
 	if (assistant->priv->logo)
 		gdk_pixbuf_unref (assistant->priv->logo);
 	if (assistant->priv->dsn_info)
-		gda_data_source_info_free (assistant->priv->dsn_info);
+		data_source_info_free (assistant->priv->dsn_info);
 
 	if (assistant->priv->create_db_op)
 		g_object_unref (assistant->priv->create_db_op);
