@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2006 The GNOME Foundation
+ * Copyright (C) 1999 - 2008 The GNOME Foundation
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -30,7 +30,8 @@
 #include <libgnomedb/gnome-db-util.h>
 #include <libgnomedb/gnome-db-basic-form.h>
 #include <glib/gi18n-lib.h>
-#include "gnome-db-dsn-spec.h"
+#include "gnome-db-provider-spec-editor.h"
+#include "gnome-db-provider-auth-editor.h"
 
 #define PARENT_TYPE GTK_TYPE_VBOX
 
@@ -38,12 +39,15 @@ struct _GnomeDbDsnEditorPrivate {
 	GtkWidget *wname;
 	GtkWidget *wprovider;
 	GtkWidget *wdesc;
-	GtkWidget *wuser;
-	GtkWidget *wpassword;
-	GtkWidget *is_global;
+	GtkWidget *is_system;
 
-	GtkWidget *expander;
+	GtkWidget *dsn_spec_expander;
 	GtkWidget *dsn_spec;
+
+	GtkWidget *dsn_auth_expander;
+	GtkWidget *dsn_auth;	
+
+	GdaDataSourceInfo *dsn_info;
 };
 
 static void gnome_db_dsn_editor_class_init (GnomeDbDsnEditorClass *klass);
@@ -89,8 +93,8 @@ field_changed_cb (GtkWidget *widget, GnomeDbDsnEditor *config)
 {
 	if (widget == config->priv->wprovider) 
 		/* replace the expander's contents */
-		gnome_db_dsn_spec_set_provider (GNOME_DB_DSN_SPEC (config->priv->dsn_spec),
-						gnome_db_provider_selector_get_selected_provider 
+		gnome_db_provider_spec_editor_set_provider (GNOME_DB_PROVIDER_SPEC_EDITOR (config->priv->dsn_spec),
+						gnome_db_provider_selector_get_provider 
 						(GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
 
 	g_signal_emit (config, gnome_db_dsn_editor_signals[CHANGED], 0, NULL);
@@ -114,9 +118,10 @@ gnome_db_dsn_editor_init (GnomeDbDsnEditor *config, GnomeDbDsnEditorClass *klass
 
 	/* allocate private structure */
 	config->priv = g_new0 (GnomeDbDsnEditorPrivate, 1);
+	config->priv->dsn_info = g_new0 (GdaDataSourceInfo, 1);
 
 	/* set up widgets */
-	table = gnome_db_new_table_widget (7, 3, FALSE);
+	table = gnome_db_new_table_widget (8, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (config), table, TRUE, TRUE, 0);
 
 	str = g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>", _("Data source _name:"));
@@ -139,11 +144,11 @@ gnome_db_dsn_editor_init (GnomeDbDsnEditor *config, GnomeDbDsnEditorClass *klass
 	label = gnome_db_new_label_widget (_("_System wide data source:"));
 	gtk_widget_show (label);
 	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	config->priv->is_global = gtk_check_button_new ();
-	gtk_widget_show (config->priv->is_global);
-	g_signal_connect (G_OBJECT (config->priv->is_global), "toggled",
+	config->priv->is_system = gtk_check_button_new ();
+	gtk_widget_show (config->priv->is_system);
+	g_signal_connect (G_OBJECT (config->priv->is_system), "toggled",
 			  G_CALLBACK (field_toggled_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->is_global, 1, 2, 1, 2,
+	gtk_table_attach (GTK_TABLE (table), config->priv->is_system, 1, 2, 1, 2,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
 	str = g_strdup_printf ("%s <span foreground='red' weight='bold'>*</span>", _("_Provider:"));
@@ -173,40 +178,32 @@ gnome_db_dsn_editor_init (GnomeDbDsnEditor *config, GnomeDbDsnEditorClass *klass
 	gtk_table_attach (GTK_TABLE (table), config->priv->wdesc, 1, 2, 3, 4,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
 
-	label = gnome_db_new_label_widget (_("_Username:"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 4, 5, GTK_FILL, GTK_FILL, 0, 0);
-	config->priv->wuser = gnome_db_new_entry_widget (0, TRUE);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wuser);
-	gtk_widget_show (config->priv->wuser);
-	g_signal_connect (G_OBJECT (config->priv->wuser), "changed",
-			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wuser, 1, 2, 4, 5,
-			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
-
-	label = gnome_db_new_label_widget (_("P_assword:"));
-	gtk_widget_show (label);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 5, 6, GTK_FILL, GTK_FILL, 0, 0);
-	config->priv->wpassword = gnome_db_new_entry_widget (0, TRUE);
-	gtk_label_set_mnemonic_widget (GTK_LABEL (label), config->priv->wpassword);
-	gtk_entry_set_visibility (GTK_ENTRY (config->priv->wpassword), FALSE);
-	gtk_widget_show (config->priv->wpassword);
-	g_signal_connect (G_OBJECT (config->priv->wpassword), "changed",
-			  G_CALLBACK (field_changed_cb), config);
-	gtk_table_attach (GTK_TABLE (table), config->priv->wpassword, 1, 2, 5, 6,
-			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
-
-	exp = gtk_expander_new (_("Connection's specific parameters"));
-	config->priv->expander = exp;
+	/* connection's spec */
+	exp = gtk_expander_new (_("Connection's parameters"));
+	config->priv->dsn_spec_expander = exp;
 	gtk_widget_show (exp);
-	gtk_table_attach (GTK_TABLE (table), config->priv->expander, 0, 2, 6, 7,
+	gtk_table_attach (GTK_TABLE (table), config->priv->dsn_spec_expander, 0, 2, 6, 7,
 			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
-	config->priv->dsn_spec = gnome_db_dsn_spec_new (gnome_db_provider_selector_get_selected_provider 
+	config->priv->dsn_spec = gnome_db_provider_spec_editor_new (gnome_db_provider_selector_get_provider 
 							(GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
 	g_signal_connect (G_OBJECT (config->priv->dsn_spec), "changed",
 			  G_CALLBACK (field_changed_cb), config);
-	gtk_container_add (GTK_CONTAINER (config->priv->expander), config->priv->dsn_spec);
+	gtk_container_add (GTK_CONTAINER (config->priv->dsn_spec_expander), config->priv->dsn_spec);
 	gtk_widget_show (config->priv->dsn_spec);
+
+	/* connection's authentication */
+	exp = gtk_expander_new (_("Authentication"));
+	config->priv->dsn_auth_expander = exp;
+	gtk_widget_show (exp);
+	gtk_table_attach (GTK_TABLE (table), config->priv->dsn_auth_expander, 0, 2, 7, 8,
+			  GTK_FILL|GTK_EXPAND, GTK_FILL, 0, 0);
+	config->priv->dsn_auth = gnome_db_provider_auth_editor_new (gnome_db_provider_selector_get_provider 
+							(GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
+	g_signal_connect (G_OBJECT (config->priv->dsn_auth), "changed",
+			  G_CALLBACK (field_changed_cb), config);
+	gtk_container_add (GTK_CONTAINER (config->priv->dsn_auth_expander), config->priv->dsn_auth);
+	gtk_widget_show (config->priv->dsn_spec);
+
 }
 
 static void
@@ -217,6 +214,11 @@ gnome_db_dsn_editor_finalize (GObject *object)
 	g_return_if_fail (GNOME_DB_IS_DSN_EDITOR (config));
 
 	/* free memory */
+	g_free (config->priv->dsn_info->provider); 
+	g_free (config->priv->dsn_info->cnc_string); 
+	g_free (config->priv->dsn_info->description);
+	g_free (config->priv->dsn_info->auth_string);
+	g_free (config->priv->dsn_info);
 	g_free (config->priv);
 
 	/* chain to parent class */
@@ -269,56 +271,93 @@ gnome_db_dsn_editor_new (void)
  *
  *
  *
- * Returns:
+ * Returns: a pointer to the currently configured DSN (do not modify)
  */
-GdaDataSourceInfo *
+const GdaDataSourceInfo *
 gnome_db_dsn_editor_get_dsn (GnomeDbDsnEditor *config)
 {
-	gint i;
 	GdaDataSourceInfo *dsn_info;
 
 	g_return_val_if_fail (GNOME_DB_IS_DSN_EDITOR (config), NULL);
+	dsn_info = config->priv->dsn_info;
 
-	dsn_info = g_new0 (GdaDataSourceInfo, 1);
+	g_free (dsn_info->provider); dsn_info->provider = NULL;
+	g_free (dsn_info->cnc_string); dsn_info->cnc_string = NULL;
+	g_free (dsn_info->description); dsn_info->description = NULL;
+	g_free (dsn_info->auth_string); dsn_info->auth_string = NULL;
 	dsn_info->name = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wname)));
-	dsn_info->provider = g_strdup (gnome_db_provider_selector_get_selected_provider 
+	dsn_info->provider = g_strdup (gnome_db_provider_selector_get_provider 
 				       (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider)));
-	dsn_info->cnc_string = gnome_db_dsn_spec_get_specs (GNOME_DB_DSN_SPEC (config->priv->dsn_spec));
+	dsn_info->cnc_string = gnome_db_provider_spec_editor_get_specs (GNOME_DB_PROVIDER_SPEC_EDITOR (config->priv->dsn_spec));
 	dsn_info->description = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wdesc)));
-	dsn_info->username = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wuser)));
-	dsn_info->password = g_strdup (gtk_entry_get_text (GTK_ENTRY (config->priv->wpassword)));
-	dsn_info->is_global = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (config->priv->is_global));
+	dsn_info->auth_string = gnome_db_provider_auth_editor_get_auth (GNOME_DB_PROVIDER_AUTH_EDITOR (config->priv->dsn_auth));
+	dsn_info->is_system = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (config->priv->is_system));
 
 	return dsn_info;
 }
 
 /**
  * gnome_db_dsn_editor_set_dsn
- * @config:
- * @dsn_info:
+ * @editor: a #GnomeDbDsnEditor widget
+ * @dsn_info: a #GdaDataSourceInfo pointer or %NULL
  *
+ *
+ * Requests that @editor update its contents with @dsn_info's contents
  */
 void
-gnome_db_dsn_editor_set_dsn (GnomeDbDsnEditor *config, const GdaDataSourceInfo *dsn_info)
+gnome_db_dsn_editor_set_dsn (GnomeDbDsnEditor *editor, const GdaDataSourceInfo *dsn_info)
 {
-	g_return_if_fail (GNOME_DB_IS_DSN_EDITOR (config));
-	g_return_if_fail (dsn_info != NULL);
+	g_return_if_fail (GNOME_DB_IS_DSN_EDITOR (editor));
 
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wname), dsn_info->name);
-	gnome_db_provider_selector_set_selected_provider (GNOME_DB_PROVIDER_SELECTOR (config->priv->wprovider), 
-							  dsn_info->provider);
-	gnome_db_dsn_spec_set_specs (GNOME_DB_DSN_SPEC (config->priv->dsn_spec), dsn_info->cnc_string);
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wdesc), dsn_info->description ? dsn_info->description : "");
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wuser), dsn_info->username ? dsn_info->username : "");
-	gtk_entry_set_text (GTK_ENTRY (config->priv->wpassword), dsn_info->password ? dsn_info->password : "");
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (config->priv->is_global), dsn_info->is_global);
-
-	if (dsn_info->is_global && !gda_config_can_modify_global_config ())
-		gtk_widget_set_sensitive (GTK_WIDGET (config), FALSE);
+	if (dsn_info) {
+		gtk_entry_set_text (GTK_ENTRY (editor->priv->wname), dsn_info->name);
+		gnome_db_provider_selector_set_provider (GNOME_DB_PROVIDER_SELECTOR (editor->priv->wprovider), 
+							 dsn_info->provider);
+		gnome_db_provider_spec_editor_set_provider (GNOME_DB_PROVIDER_SPEC_EDITOR (editor->priv->dsn_spec), dsn_info->provider);
+		gnome_db_provider_spec_editor_set_specs (GNOME_DB_PROVIDER_SPEC_EDITOR (editor->priv->dsn_spec), dsn_info->cnc_string);
+		gtk_entry_set_text (GTK_ENTRY (editor->priv->wdesc), dsn_info->description ? dsn_info->description : "");
+		gnome_db_provider_auth_editor_set_provider (GNOME_DB_PROVIDER_AUTH_EDITOR (editor->priv->dsn_auth), dsn_info->provider);
+		gnome_db_provider_auth_editor_set_auth (GNOME_DB_PROVIDER_AUTH_EDITOR (editor->priv->dsn_auth), dsn_info->auth_string);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->priv->is_system), dsn_info->is_system);
+		
+		if (dsn_info->is_system && !gda_config_can_modify_system_config ()) {
+			gtk_widget_set_sensitive (editor->priv->wname, FALSE);
+			gtk_widget_set_sensitive (editor->priv->wprovider, FALSE);
+			gtk_widget_set_sensitive (editor->priv->wdesc, FALSE);
+			gtk_widget_set_sensitive (editor->priv->dsn_spec, FALSE);
+			gtk_widget_set_sensitive (editor->priv->dsn_auth, FALSE);
+			gtk_widget_set_sensitive (editor->priv->is_system, FALSE);
+		}
+		else {
+			gtk_widget_set_sensitive (editor->priv->wname, FALSE);
+			gtk_widget_set_sensitive (editor->priv->wprovider, TRUE);
+			gtk_widget_set_sensitive (editor->priv->wdesc, TRUE);
+			gtk_widget_set_sensitive (editor->priv->dsn_spec, TRUE);
+			gtk_widget_set_sensitive (editor->priv->dsn_auth, TRUE);
+			gtk_widget_set_sensitive (editor->priv->is_system, gda_config_can_modify_system_config ());
+		}
+		gtk_widget_show (editor->priv->dsn_spec_expander);
+		if (gda_config_dsn_needs_authentication (dsn_info->name))
+			gtk_widget_show (editor->priv->dsn_auth_expander);
+		else
+			gtk_widget_hide (editor->priv->dsn_auth_expander);
+	}
 	else {
-		gtk_widget_set_sensitive (GTK_WIDGET (config), TRUE);
-		gtk_widget_set_sensitive (config->priv->wname, FALSE); /* FIXME */
-		gtk_widget_set_sensitive (config->priv->is_global, 
-					  gda_config_can_modify_global_config ());
+		gtk_entry_set_text (GTK_ENTRY (editor->priv->wname), "");
+		gnome_db_provider_selector_set_provider (GNOME_DB_PROVIDER_SELECTOR (editor->priv->wprovider), NULL);
+		gnome_db_provider_spec_editor_set_provider (GNOME_DB_PROVIDER_SPEC_EDITOR (editor->priv->dsn_spec), NULL);
+		gtk_entry_set_text (GTK_ENTRY (editor->priv->wdesc), "");
+		gnome_db_provider_auth_editor_set_provider (GNOME_DB_PROVIDER_AUTH_EDITOR (editor->priv->dsn_auth), NULL);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (editor->priv->is_system), FALSE);
+		
+		gtk_widget_set_sensitive (editor->priv->wname, FALSE);
+		gtk_widget_set_sensitive (editor->priv->wprovider, FALSE);
+		gtk_widget_set_sensitive (editor->priv->wdesc, FALSE);
+		gtk_widget_set_sensitive (editor->priv->dsn_spec, FALSE);
+		gtk_widget_set_sensitive (editor->priv->dsn_auth, FALSE);
+		gtk_widget_set_sensitive (editor->priv->is_system, FALSE);
+
+		gtk_widget_hide (editor->priv->dsn_spec_expander);
+		gtk_widget_hide (editor->priv->dsn_auth_expander);
 	}
 }
