@@ -53,7 +53,7 @@ struct _GnomeDbGridClass
 /* 
  * Generic widget's methods 
  */
-GType             gnome_db_grid_get_type            (void);
+GType             gnome_db_grid_get_type            (void) G_GNUC_CONST;
 
 GtkWidget        *gnome_db_grid_new                 (GdaDataModel *model);
 GList            *gnome_db_grid_get_selection       (GnomeDbGrid *grid);
