@@ -1,5 +1,5 @@
 /* GNOME DB library
- * Copyright (C) 1999 - 2007 The GNOME Foundation.
+ * Copyright (C) 1999 - 2008 The GNOME Foundation.
  *
  * AUTHORS:
  *      Rodrigo Moya <rodrigo@gnome-db.org>
@@ -23,13 +23,15 @@
 
 #include "gnome-db-decl.h"
 #include <libgda/gda-config.h>
-#include <libgnomedb/gnome-db-data-source-selector.h>
+#include <libgnomedb/gnome-db-dsn-selector.h>
 #ifdef HAVE_GTKTWOTEN
 #include <libgnomedb/gnome-db-dsn-assistant.h>
 #endif
 #include <glib/gi18n-lib.h>
 #include <libgnomedb/gnome-db-login.h>
 #include <libgnomedb/gnome-db-util.h>
+#include <libgnomedb/gnome-db-basic-form.h>
+#include <libgnomedb/gnome-db-provider-auth-editor.h>
 #include <gtk/gtklabel.h>
 #include <gtk/gtktable.h>
 #include <gtk/gtkstock.h>
@@ -41,8 +43,8 @@ struct _GnomeDbLoginPrivate {
 	GtkWidget *create_dsn_button;
 	gboolean   hiding_button;
 	gboolean   hiding_dsn;
-	GtkWidget *username_entry;
-	GtkWidget *password_entry;
+
+	GtkWidget *auth_widget;
 };
 
 static void gnome_db_login_class_init   (GnomeDbLoginClass *klass);
@@ -68,6 +70,16 @@ enum {
 
 static GObjectClass *parent_class = NULL;
 
+/* signals */
+enum
+{
+        DSN_CHANGED,
+        LAST_SIGNAL
+};
+
+static gint gnome_db_login_signals[LAST_SIGNAL] = { 0 };
+
+
 #ifdef HAVE_GTKTWOTEN
 static void
 assistant_finished_cb (GnomeDbDsnAssistant *assistant, gboolean error, GnomeDbLogin *login);
@@ -75,11 +87,7 @@ static void
 add_dsn_cb (GtkButton *button, GnomeDbLogin *login);
 #endif
 static void
-dsn_entry_changed_cb (GnomeDbDataSourceSelector *sel, GnomeDbLogin *login);
-
-/*
- * Private functions
- */
+dsn_entry_changed_cb (GnomeDbDsnSelector *sel, GnomeDbLogin *login);
 
 /*
  * GnomeDbLogin class implementation
@@ -92,6 +100,17 @@ gnome_db_login_class_init (GnomeDbLoginClass *klass)
 
 	parent_class = g_type_class_peek_parent (klass);
 
+	gnome_db_login_signals[DSN_CHANGED] =
+                g_signal_new ("dsn-changed",
+                              G_TYPE_FROM_CLASS (object_class),
+                              G_SIGNAL_RUN_FIRST,
+                              G_STRUCT_OFFSET (GnomeDbLoginClass, dsn_changed),
+                              NULL, NULL,
+                              g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
+                              0);
+
+	klass->dsn_changed = NULL;
+
 	object_class->set_property = gnome_db_login_set_property;
 	object_class->get_property = gnome_db_login_get_property;
 	object_class->finalize = gnome_db_login_finalize;
@@ -129,8 +148,7 @@ gnome_db_login_init (GnomeDbLogin *login, GnomeDbLoginClass *klass)
 	login->priv->hiding_button = TRUE;
 	login->priv->hiding_dsn = FALSE;
 	
-	/*construct the widget*/
-	
+	/* construct the widget */
 	table = gnome_db_new_table_widget (3, 3, FALSE);
 	gtk_box_pack_start (GTK_BOX (login), table, TRUE, TRUE, 0);
 	
@@ -143,7 +161,8 @@ gnome_db_login_init (GnomeDbLogin *login, GnomeDbLoginClass *klass)
 	login->priv->dsn_label = label;
 	
 	/* Create the DSN selector*/
-	login->priv->dsn_entry = gnome_db_data_source_selector_new (login->priv->dsn_name);
+	login->priv->dsn_entry = gnome_db_dsn_selector_new ();
+	gnome_db_dsn_selector_set_dsn (GNOME_DB_DSN_SELECTOR (login->priv->dsn_entry), login->priv->dsn_name);
 	gtk_widget_show (login->priv->dsn_entry); /* Show the DSN selector */
 	gtk_table_attach (GTK_TABLE (table), login->priv->dsn_entry, 1, 2, 0, 1,
 			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
@@ -162,27 +181,10 @@ gnome_db_login_init (GnomeDbLogin *login, GnomeDbLoginClass *klass)
 	login->priv->create_dsn_button = NULL;
 #endif
 			  
-	/* Create the Username Entry*/
-	label = gnome_db_new_label_widget (_("Username:"));
-        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 1, 2, GTK_FILL, GTK_FILL, 0, 0);
-	login->priv->username_entry = gnome_db_new_entry_widget (0, TRUE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->username_entry, 1, 3, 1, 2,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);			
-
-	/* Create the Password Entry*/
-	label = gnome_db_new_label_widget (_("Password:"));
-        gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
-	gtk_label_set_selectable (GTK_LABEL (label), FALSE);
-        gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
-	gtk_table_attach (GTK_TABLE (table), label, 0, 1, 2, 3, GTK_FILL, GTK_FILL, 0, 0);
-	login->priv->password_entry = gnome_db_new_entry_widget (0, TRUE);
-	gtk_entry_set_visibility (GTK_ENTRY (login->priv->password_entry), FALSE);
-	gtk_table_attach (GTK_TABLE (table), login->priv->password_entry, 1, 3, 2, 3,
-			  GTK_FILL | GTK_EXPAND | GTK_SHRINK, GTK_FILL, 0, 0);
-
+	/* Create the authentication part */
+	login->priv->auth_widget = gnome_db_provider_auth_editor_new (NULL);
+	gtk_table_attach_defaults (GTK_TABLE (table), login->priv->auth_widget, 0, 3, 1, 2);
+	gtk_widget_show (login->priv->auth_widget);
 }
 
 static void
@@ -198,10 +200,10 @@ gnome_db_login_set_property (GObject *object,
 	switch (param_id) {
 	
 	case PROP_HIDING_BUTTON:
-		gnome_db_login_set_enable_create_button(login, g_value_get_boolean(value));
+		gnome_db_login_set_enable_create_button(login, !g_value_get_boolean(value));
 		break;
 	case PROP_HIDING_DSN:
-		gnome_db_login_set_show_dsn_selector(login, g_value_get_boolean(value));
+		gnome_db_login_set_show_dsn_selector(login, !g_value_get_boolean(value));
 		break;
 	default :
 		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, param_id, pspec);
@@ -327,25 +329,17 @@ add_dsn_cb (GtkButton *button, GnomeDbLogin *login)
 #endif
 
 static void
-dsn_entry_changed_cb (GnomeDbDataSourceSelector *sel, GnomeDbLogin *login)
+dsn_entry_changed_cb (GnomeDbDsnSelector *sel, GnomeDbLogin *login)
 {
 	gchar *dsn;
 	GdaDataSourceInfo *info = NULL;
         
-        dsn = gnome_db_data_source_selector_get_dsn(sel);
+        dsn = gnome_db_dsn_selector_get_dsn (sel);
         
-	info = gda_config_find_data_source (dsn);
+	info = gda_config_get_dsn (dsn);
 
-	gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), "");
-	gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), "");
-
-	if (info) {
-		if (info->username)
-			gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), info->username);
-		if (info->password)
-			gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), info->password);
-		gda_data_source_info_free (info);
-	}
+	if (info)
+		TO_IMPLEMENT;
 }
 
 /**
@@ -381,7 +375,7 @@ gnome_db_login_get_dsn (GnomeDbLogin *login)
 	if (login->priv->dsn_name)
 		return (const gchar *) login->priv->dsn_name;
 
-	return gnome_db_data_source_selector_get_dsn (GNOME_DB_DATA_SOURCE_SELECTOR (login->priv->dsn_entry));
+	return gnome_db_dsn_selector_get_dsn (GNOME_DB_DSN_SELECTOR (login->priv->dsn_entry));
 }
 
 /**
@@ -397,20 +391,52 @@ gnome_db_login_set_dsn (GnomeDbLogin *login, const gchar *dsn)
 	GdaDataSourceInfo *src;
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
 
-	src = gda_config_find_data_source (dsn);
+	src = gda_config_get_dsn (dsn);
 	if (!src)
 		g_warning (_("Datasource '%s' is not declared"), dsn);
 	else {
-		if (src->username)
-			gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), src->username);
-		if (src->password)
-			gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), src->password);
-		gda_data_source_info_free (src);
-		gnome_db_data_source_selector_set_dsn (GNOME_DB_DATA_SOURCE_SELECTOR (login->priv->dsn_entry), 
-						       dsn);
+		gnome_db_dsn_selector_set_dsn (GNOME_DB_DSN_SELECTOR (login->priv->dsn_entry), 
+					       dsn);
+		gnome_db_provider_auth_editor_set_provider 
+			(GNOME_DB_PROVIDER_AUTH_EDITOR (login->priv->auth_widget), src->provider);
+		g_signal_emit (login, gnome_db_login_signals[DSN_CHANGED], 0);
 	}
 }
 
+/**
+ * gnome_db_login_get_auth
+ * @login: a #GnomeDbLogin widget
+ *
+ * Returns: the authentication string
+ */
+const gchar *
+gnome_db_login_get_auth (GnomeDbLogin *login)
+{
+	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), NULL);
+	if (! login->priv->auth_widget)
+		return NULL;
+
+	return gnome_db_provider_auth_editor_get_auth (GNOME_DB_PROVIDER_AUTH_EDITOR (login->priv->auth_widget));
+}
+
+/**
+ * gnome_db_login_set_auth
+ * @login: a #GnomeDbLogin widget
+ * @auth: an authentication string, or %NULL
+ *
+ * Update the authentication part of @login using @auth
+ */
+void
+gnome_db_login_set_auth (GnomeDbLogin *login, const gchar *auth)
+{
+	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
+	if (! login->priv->auth_widget)
+		return;
+
+	gnome_db_provider_auth_editor_set_auth (GNOME_DB_PROVIDER_AUTH_EDITOR (login->priv->auth_widget),
+						auth);
+}
+
 /**
  * gnome_db_login_get_username
  * @login: A #GnomeDbLogin widget.
@@ -423,7 +449,8 @@ const gchar *
 gnome_db_login_get_username (GnomeDbLogin *login)
 {
 	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), NULL);
-	return gtk_entry_get_text (GTK_ENTRY (login->priv->username_entry));
+	TO_IMPLEMENT;
+	return NULL;
 }
 
 /**
@@ -437,7 +464,7 @@ void
 gnome_db_login_set_username (GnomeDbLogin *login, const gchar *username)
 {
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
-	gtk_entry_set_text (GTK_ENTRY (login->priv->username_entry), username);
+	TO_IMPLEMENT;
 }
 
 /**
@@ -452,7 +479,8 @@ const gchar *
 gnome_db_login_get_password (GnomeDbLogin *login)
 {
 	g_return_val_if_fail (GNOME_DB_IS_LOGIN (login), NULL);
-	return gtk_entry_get_text (GTK_ENTRY (login->priv->password_entry));
+	TO_IMPLEMENT;
+	return NULL;
 }
 
 /**
@@ -466,7 +494,7 @@ void
 gnome_db_login_set_password (GnomeDbLogin *login, const gchar *password)
 {
 	g_return_if_fail (GNOME_DB_IS_LOGIN (login));
-	gtk_entry_set_text (GTK_ENTRY (login->priv->password_entry), password);
+	TO_IMPLEMENT;
 }
 
 /**
