@@ -1,6 +1,6 @@
 /* gnome-db-server-operation.c
  *
- * Copyright (C) 2006 - 2007 Vivien Malerba
+ * Copyright (C) 2006 - 2008 Vivien Malerba
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -224,7 +224,7 @@ gnome_db_server_operation_init (GnomeDbServerOperation * wid)
 
 /**
  * gnome_db_server_operation_new
- * @paramlist: a #GdaParameterList structure
+ * @paramlist: a #GdaSet structure
  *
  * Creates a new #GnomeDbServerOperation widget using all the parameters provided in @paramlist.
  *
@@ -421,7 +421,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 	/* generic widget rendering */
 	switch (info_node->type) {
 	case GDA_SERVER_OPERATION_NODE_PARAMLIST: {
-		GdaParameterList *plist;
+		GdaSet *plist;
 
 		plist = info_node->plist;
 		plwid = gnome_db_basic_form_new (plist);
@@ -430,7 +430,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 	       
 		if (section_str) {
 			const gchar *name;
-			name = gda_object_get_name (GDA_OBJECT (plist));
+			name = g_object_get_data (G_OBJECT (plist), "name");
 			if (name && *name)
 				*section_str = g_strdup_printf ("<b>%s:</b>", name);
 			else
@@ -439,12 +439,12 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		if (label_widgets) {
 			GSList *params;
 			
-			params = plist->parameters;
+			params = plist->holders;
 			while (params) {
 				GtkWidget *label_entry;
 
 				label_entry = gnome_db_basic_form_get_label_widget (GNOME_DB_BASIC_FORM (plwid),
-										    GDA_PARAMETER (params->data));
+										    GDA_HOLDER (params->data));
 				if (label_entry && !g_slist_find (*label_widgets, label_entry))
 					*label_widgets = g_slist_prepend (*label_widgets, label_entry);
 				params = params->next;
@@ -491,7 +491,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		
 		if (section_str)
 			*section_str = g_strdup_printf ("<b>%s:</b>", 
-						       gda_object_get_name (GDA_OBJECT (model)));
+							(gchar*) g_object_get_data (G_OBJECT (model), "name"));
 
 		if (label_widgets) {
 			GtkWidget *label_entry;
@@ -499,12 +499,13 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 			gchar *str;
 
 			if (info_node->status == GDA_SERVER_OPERATION_STATUS_REQUIRED) {
-				str = g_strdup_printf ("<b>%s:</b>", gda_object_get_name (GDA_OBJECT (model)));
+				str = g_strdup_printf ("<b>%s:</b>", 
+						       (gchar*) g_object_get_data (G_OBJECT (model), "name"));
 				label_entry = gtk_label_new (str);
 				gtk_label_set_use_markup (GTK_LABEL (label_entry), TRUE);
 			}
 			else {
-				str = g_strdup_printf ("%s:", gda_object_get_name (GDA_OBJECT (model)));
+				str = g_strdup_printf ("%s:", (gchar*) g_object_get_data (G_OBJECT (model), "name"));
 				label_entry = gtk_label_new (str);
 			}
 			g_free (str);
@@ -513,7 +514,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 			evbox = gtk_event_box_new ();
 			gtk_container_add (GTK_CONTAINER (evbox), label_entry);
 			gtk_widget_show (label_entry);
-			str = (gchar *) gda_object_get_description (GDA_OBJECT (model));
+			str = (gchar *) g_object_get_data (G_OBJECT (model), "descr");
 			if (str && *str) 
 				gtk_tooltips_set_tip (form->priv->tooltips, evbox, str, NULL);
 
@@ -525,13 +526,13 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 		break;
 	}
 	case GDA_SERVER_OPERATION_NODE_PARAM: {
-		GdaParameterList *plist;
-		GdaParameter *param;
+		GdaSet *plist;
+		GdaHolder *param;
 		GSList *list;
 
 		param = info_node->param;
 		list = g_slist_append (NULL, param);
-		plist = gda_parameter_list_new (list);
+		plist = gda_set_new (list);
 		g_slist_free (list);
 		plwid = gnome_db_basic_form_new (plist);
 		gnome_db_basic_form_show_entry_actions (GNOME_DB_BASIC_FORM (plwid),
@@ -541,7 +542,7 @@ fill_create_widget (GnomeDbServerOperation *form, const gchar *path, gchar **sec
 
 		if (section_str)
 			*section_str = g_strdup_printf ("<b>%s:</b>", 
-							gda_object_get_name (GDA_OBJECT (param)));
+							(gchar*) g_object_get_data (G_OBJECT (param), "name"));
 		if (label_widgets) {
 			GtkWidget *label_entry;
 
